@@ -11,12 +11,14 @@
 #include <libfilezilla/event_loop.hpp>
 #include <libfilezilla/local_filesys.hpp>
 #include <libfilezilla/process.hpp>
+#include <libfilezilla/thread_pool.hpp>
 
-#include <wx/log.h>
+#include <wx/string.h>
 
+#include <algorithm>
 #include <cwchar>
 
-#define FZSFTP_PROTOCOL_VERSION 6
+#define FZSFTP_PROTOCOL_VERSION 7
 
 struct sftp_message
 {
@@ -49,7 +51,7 @@ enum filetransferStates
 	filetransfer_chmtime
 };
 
-class CSftpInputThread final : public fz::thread
+class CSftpInputThread final
 {
 public:
 	CSftpInputThread(CSftpControlSocket* pOwner, fz::process& proc)
@@ -58,9 +60,17 @@ public:
 	{
 	}
 
-	virtual ~CSftpInputThread()
+	~CSftpInputThread()
 	{
-		join();
+		thread_.join();
+	}
+
+	bool spawn(fz::thread_pool & pool)
+	{
+		if (!thread_) {
+			thread_ = pool.spawn([this]() { entry(); });
+		}
+		return thread_.operator bool();
 	}
 
 protected:
@@ -102,15 +112,15 @@ protected:
 
 		buffer[len] = 0;
 
-		const wxString line = m_pOwner->ConvToLocal(buffer, len + 1);
+		std::wstring const line = m_pOwner->ConvToLocal(buffer, len + 1);
 		if (len && line.empty()) {
 			error = L"Failed to convert reply to local character set.";
 		}
 
-		return line.ToStdWstring();
+		return line;
 	}
 
-	virtual void entry()
+	void entry()
 	{
 		std::wstring error;
 		while (error.empty()) {
@@ -145,6 +155,7 @@ protected:
 			case sftpEvent::Done:
 			case sftpEvent::Error:
 			case sftpEvent::Verbose:
+			case sftpEvent::Info:
 			case sftpEvent::Status:
 			case sftpEvent::Transfer:
 			case sftpEvent::AskPassword:
@@ -190,6 +201,8 @@ protected:
 
 	fz::process& process_;
 	CSftpControlSocket* m_pOwner;
+
+	fz::async_task thread_;
 };
 
 class CSftpDeleteOpData final : public COpData
@@ -248,7 +261,7 @@ public:
 	{
 	}
 
-	wxString lastChallenge;
+	std::wstring lastChallenge;
 	CInteractiveLoginNotification::type lastChallengeType{CInteractiveLoginNotification::interactive};
 	bool criticalFailure{};
 
@@ -277,10 +290,6 @@ int CSftpControlSocket::Connect(const CServer &server)
 	delete m_pCurrentServer;
 	m_pCurrentServer = new CServer(server);
 
-	if (CServerCapabilities::GetCapability(*m_pCurrentServer, timezone_offset) == unknown) {
-		CServerCapabilities::SetCapability(*m_pCurrentServer, timezone_offset, yes, 0);
-	}
-
 	CSftpConnectOpData* pData = new CSftpConnectOpData;
 	m_pCurOpData = pData;
 
@@ -292,6 +301,17 @@ int CSftpControlSocket::Connect(const CServer &server)
 	else {
 		pData->keyfiles_ = fz::strtok(engine_.GetOptions().GetOption(OPTION_SFTP_KEYFILES), L"\r\n");
 	}
+
+	pData->keyfiles_.erase(
+		std::remove_if(pData->keyfiles_.begin(), pData->keyfiles_.end(),
+			[this](std::wstring const& keyfile) {
+				if (fz::local_filesys::get_file_type(fz::to_native(keyfile), true) != fz::local_filesys::file) {
+					LogMessage(MessageType::Status, _("Skipping non-existing key file \"%s\""), keyfile);
+					return true;
+				}
+				return false;
+		}), pData->keyfiles_.end());
+
 	pData->keyfile_ = pData->keyfiles_.cbegin();
 
 	m_pProcess = new fz::process();
@@ -314,7 +334,7 @@ int CSftpControlSocket::Connect(const CServer &server)
 	}
 
 	m_pInputThread = new CSftpInputThread(this, *m_pProcess);
-	if (!m_pInputThread->run()) {
+	if (!m_pInputThread->spawn(engine_.GetThreadPool())) {
 		LogMessage(MessageType::Debug_Warning, _T("Thread creation failed"));
 		delete m_pInputThread;
 		m_pInputThread = 0;
@@ -325,7 +345,7 @@ int CSftpControlSocket::Connect(const CServer &server)
 	return FZ_REPLY_WOULDBLOCK;
 }
 
-int CSftpControlSocket::ConnectParseResponse(bool successful, const wxString& reply)
+int CSftpControlSocket::ConnectParseResponse(bool successful, std::wstring const& reply)
 {
 	LogMessage(MessageType::Debug_Verbose, _T("CSftpControlSocket::ConnectParseResponse(%s)"), reply);
 
@@ -350,7 +370,7 @@ int CSftpControlSocket::ConnectParseResponse(bool successful, const wxString& re
 	switch (pData->opState)
 	{
 	case connect_init:
-		if (reply != wxString::Format(_T("fzSftp started, protocol_version=%d"), FZSFTP_PROTOCOL_VERSION)) {
+		if (reply != fz::sprintf(L"fzSftp started, protocol_version=%d", FZSFTP_PROTOCOL_VERSION)) {
 			LogMessage(MessageType::Error, _("fzsftp belongs to a different version of FileZilla"));
 			DoClose(FZ_REPLY_INTERNALERROR);
 			return FZ_REPLY_ERROR;
@@ -394,16 +414,14 @@ int CSftpControlSocket::ConnectParseResponse(bool successful, const wxString& re
 int CSftpControlSocket::ConnectSend()
 {
 	LogMessage(MessageType::Debug_Verbose, _T("CSftpControlSocket::ConnectSend()"));
-	if (!m_pCurOpData)
-	{
+	if (!m_pCurOpData) {
 		LogMessage(__TFILE__, __LINE__, this, MessageType::Debug_Info, _T("Empty m_pCurOpData"));
 		DoClose(FZ_REPLY_INTERNALERROR);
 		return FZ_REPLY_ERROR;
 	}
 
 	CSftpConnectOpData *pData = static_cast<CSftpConnectOpData *>(m_pCurOpData);
-	if (!pData)
-	{
+	if (!pData) {
 		LogMessage(__TFILE__, __LINE__, this, MessageType::Debug_Warning, _T("m_pCurOpData of wrong type"));
 		DoClose(FZ_REPLY_INTERNALERROR);
 		return FZ_REPLY_ERROR;
@@ -461,19 +479,23 @@ int CSftpControlSocket::ConnectSend()
 		return FZ_REPLY_ERROR;
 	}
 
-	if (res)
+	if (res) {
 		return FZ_REPLY_WOULDBLOCK;
-	else
+	}
+	else {
 		return FZ_REPLY_ERROR;
+	}
 }
 
 void CSftpControlSocket::OnSftpEvent(sftp_message const& message)
 {
-	if (!m_pCurrentServer)
+	if (!m_pCurrentServer) {
 		return;
+	}
 
-	if (!m_pInputThread)
+	if (!m_pInputThread) {
 		return;
+	}
 
 	switch (message.type)
 	{
@@ -502,6 +524,9 @@ void CSftpControlSocket::OnSftpEvent(sftp_message const& message)
 	case sftpEvent::Verbose:
 		LogMessageRaw(MessageType::Debug_Info, message.text[0]);
 		break;
+	case sftpEvent::Info:
+		LogMessageRaw(MessageType::Command, message.text[0]); // Not exactly the right message type, but it's a silent one.
+		break;
 	case sftpEvent::Status:
 		LogMessageRaw(MessageType::Status, message.text[0]);
 		break;
@@ -542,8 +567,8 @@ void CSftpControlSocket::OnSftpEvent(sftp_message const& message)
 	case sftpEvent::AskHostkey:
 	case sftpEvent::AskHostkeyChanged:
 		{
-			long port = 0;
-			if (!wxString(message.text[1]).ToLong(&port)) {
+			auto port = fz::to_integral<int>(message.text[1]);
+			if (port <= 0 || port > 65535) {
 				DoClose(FZ_REPLY_INTERNALERROR);
 				break;
 			}
@@ -562,7 +587,7 @@ void CSftpControlSocket::OnSftpEvent(sftp_message const& message)
 		else {
 			CSftpConnectOpData *pData = static_cast<CSftpConnectOpData*>(m_pCurOpData);
 
-			wxString const challengeIdentifier = m_requestPreamble + _T("\n") + m_requestInstruction + _T("\n") + message.text[0];
+			std::wstring const challengeIdentifier = m_requestPreamble + _T("\n") + m_requestInstruction + _T("\n") + message.text[0];
 
 			CInteractiveLoginNotification::type t = CInteractiveLoginNotification::interactive;
 			if (m_pCurrentServer->GetLogonType() == INTERACTIVE || m_requestPreamble == _T("SSH key passphrase")) {
@@ -570,7 +595,7 @@ void CSftpControlSocket::OnSftpEvent(sftp_message const& message)
 					t = CInteractiveLoginNotification::keyfile;
 				}
 
-				wxString challenge;
+				std::wstring challenge;
 				if (!m_requestPreamble.empty() && t != CInteractiveLoginNotification::keyfile) {
 					challenge += m_requestPreamble + _T("\n");
 				}
@@ -598,7 +623,7 @@ void CSftpControlSocket::OnSftpEvent(sftp_message const& message)
 					return;
 				}
 
-				std::wstring const pass = m_pCurrentServer->GetPass().ToStdWstring();
+				std::wstring const pass = m_pCurrentServer->GetPass();
 				std::wstring show = _T("Pass: ");
 				show.append(pass.size(), '*');
 				SendCommand(pass, show);
@@ -681,20 +706,21 @@ bool CSftpControlSocket::SendCommand(std::wstring const& cmd, std::wstring const
 	return AddToStream(cmd + _T("\n"));
 }
 
-bool CSftpControlSocket::AddToStream(const wxString& cmd, bool force_utf8)
+bool CSftpControlSocket::AddToStream(std::wstring const& cmd, bool force_utf8)
 {
 	if (!m_pProcess) {
+		ResetOperation(FZ_REPLY_INTERNALERROR);
 		return false;
 	}
 
-	wxCharBuffer const str = ConvToServer(cmd, force_utf8);
-	if (!str) {
+	std::string const str = ConvToServer(cmd, force_utf8);
+	if (str.empty()) {
 		LogMessage(MessageType::Error, _("Could not convert command to server encoding"));
+		ResetOperation(FZ_REPLY_OK);
 		return false;
 	}
 
-	unsigned int len = strlen(str);
-	return m_pProcess->write(str, len);
+	return m_pProcess->write(str);
 }
 
 bool CSftpControlSocket::SetAsyncRequestReply(CAsyncRequestNotification *pNotification)
@@ -727,22 +753,26 @@ bool CSftpControlSocket::SetAsyncRequestReply(CAsyncRequestNotification *pNotifi
 
 			CHostKeyNotification *pHostKeyNotification = static_cast<CHostKeyNotification *>(pNotification);
 			std::wstring show;
-			if (requestId == reqId_hostkey)
-				show = _("Trust new Hostkey:").ToStdWstring();
-			else
-				show = _("Trust changed Hostkey:").ToStdWstring();
+			if (requestId == reqId_hostkey) {
+				show = _("Trust new Hostkey:");
+			}
+			else {
+				show = _("Trust changed Hostkey:");
+			}
 			show += ' ';
 			if (!pHostKeyNotification->m_trust) {
-				SendCommand(std::wstring(), show + _("No").ToStdWstring());
+				SendCommand(std::wstring(), show + _("No"));
 				if (m_pCurOpData && m_pCurOpData->opId == Command::connect) {
 					CSftpConnectOpData *pData = static_cast<CSftpConnectOpData *>(m_pCurOpData);
 					pData->criticalFailure = true;
 				}
 			}
-			else if (pHostKeyNotification->m_alwaysTrust)
-				SendCommand(L"y", show + _("Yes").ToStdWstring());
-			else
-				SendCommand(L"n", show + _("Once").ToStdWstring());
+			else if (pHostKeyNotification->m_alwaysTrust) {
+				SendCommand(L"y", show + _("Yes"));
+			}
+			else {
+				SendCommand(L"n", show + _("Once"));
+			}
 		}
 		break;
 	case reqId_interactiveLogin:
@@ -753,7 +783,7 @@ bool CSftpControlSocket::SetAsyncRequestReply(CAsyncRequestNotification *pNotifi
 				DoClose(FZ_REPLY_CANCELED);
 				return false;
 			}
-			std::wstring const pass = pInteractiveLoginNotification->server.GetPass().ToStdWstring();
+			std::wstring const pass = pInteractiveLoginNotification->server.GetPass();
 			m_pCurrentServer->SetUser(m_pCurrentServer->GetUser(), pass);
 			std::wstring show = L"Pass: ";
 			show.append(pass.size(), '*');
@@ -773,10 +803,6 @@ class CSftpListOpData final : public COpData
 public:
 	CSftpListOpData()
 		: COpData(Command::list)
-		, pParser()
-		, refresh()
-		, fallback_to_current()
-		, mtime_index()
 	{
 	}
 
@@ -785,18 +811,18 @@ public:
 		delete pParser;
 	}
 
-	CDirectoryListingParser* pParser;
+	CDirectoryListingParser* pParser{};
 
 	CServerPath path;
-	wxString subDir;
+	std::wstring subDir;
 
 	// Set to true to get a directory listing even if a cache
 	// lookup can be made after finding out true remote directory
-	bool refresh;
-	bool fallback_to_current;
+	bool refresh{};
+	bool fallback_to_current{};
 
 	CDirectoryListing directoryListing;
-	int mtime_index;
+	int mtime_index{};
 
 	fz::monotonic_clock m_time_before_locking;
 };
@@ -806,11 +832,10 @@ enum listStates
 	list_init = 0,
 	list_waitcwd,
 	list_waitlock,
-	list_list,
-	list_mtime
+	list_list
 };
 
-int CSftpControlSocket::List(CServerPath path /*=CServerPath()*/, wxString subDir /*=_T("")*/, int flags /*=0*/)
+int CSftpControlSocket::List(CServerPath path, std::wstring const& subDir, int flags)
 {
 	CServerPath newPath = m_CurrentPath;
 	if (!path.empty()) {
@@ -857,7 +882,7 @@ int CSftpControlSocket::List(CServerPath path /*=CServerPath()*/, wxString subDi
 	return ParseSubcommandResult(FZ_REPLY_OK);
 }
 
-int CSftpControlSocket::ListParseResponse(bool successful, const wxString& reply)
+int CSftpControlSocket::ListParseResponse(bool successful, std::wstring const& reply)
 {
 	LogMessage(MessageType::Debug_Verbose, _T("CSftpControlSocket::ListParseResponse(%s)"), reply);
 
@@ -875,78 +900,19 @@ int CSftpControlSocket::ListParseResponse(bool successful, const wxString& reply
 	}
 
 	if (pData->opState == list_list) {
-		if (!successful)
-		{
+		if (!successful) {
 			ResetOperation(FZ_REPLY_ERROR);
 			return FZ_REPLY_ERROR;
 		}
 
-		if (!pData->pParser)
-		{
+		if (!pData->pParser) {
 			LogMessage(__TFILE__, __LINE__, this, MessageType::Debug_Warning, _T("pData->pParser is 0"));
 			ResetOperation(FZ_REPLY_INTERNALERROR);
 			return FZ_REPLY_ERROR;
 		}
 
 		pData->directoryListing = pData->pParser->Parse(m_CurrentPath);
-
-		int res = ListCheckTimezoneDetection();
-		if (res != FZ_REPLY_OK)
-			return res;
-
 		engine_.GetDirectoryCache().Store(pData->directoryListing, *m_pCurrentServer);
-
-		engine_.SendDirectoryListingNotification(m_CurrentPath, !pData->pNextOpData, true, false);
-
-		ResetOperation(FZ_REPLY_OK);
-		return FZ_REPLY_OK;
-	}
-	else if (pData->opState == list_mtime) {
-		if (successful && !reply.empty()) {
-			time_t seconds = 0;
-			bool parsed = true;
-			for (unsigned int i = 0; i < reply.Len(); ++i) {
-				wxChar c = reply[i];
-				if (c < '0' || c > '9') {
-					parsed = false;
-					break;
-				}
-				seconds *= 10;
-				seconds += c - '0';
-			}
-			if (parsed) {
-				fz::datetime date(seconds, fz::datetime::seconds);
-				if (!date.empty()) {
-					wxASSERT(pData->directoryListing[pData->mtime_index].has_date());
-					fz::datetime listTime = pData->directoryListing[pData->mtime_index].time;
-					listTime -= fz::duration::from_minutes(m_pCurrentServer->GetTimezoneOffset());
-
-					int serveroffset = static_cast<int>((date - listTime).get_seconds());
-					if (!pData->directoryListing[pData->mtime_index].has_seconds()) {
-						// Round offset to full minutes
-						if (serveroffset < 0)
-							serveroffset -= 59;
-						serveroffset -= serveroffset % 60;
-					}
-
-					LogMessage(MessageType::Status, _("Timezone offset of server is %d seconds."), -serveroffset);
-
-					fz::duration span = fz::duration::from_seconds(serveroffset);
-					const int count = pData->directoryListing.GetCount();
-					for (int i = 0; i < count; ++i) {
-						CDirentry& entry = pData->directoryListing[i];
-						entry.time += span;
-					}
-
-					// TODO: Correct cached listings
-
-					CServerCapabilities::SetCapability(*m_pCurrentServer, timezone_offset, yes, serveroffset);
-				}
-			}
-		}
-
-		engine_.GetDirectoryCache().Store(pData->directoryListing, *m_pCurrentServer);
-
 		engine_.SendDirectoryListingNotification(m_CurrentPath, !pData->pNextOpData, true, false);
 
 		ResetOperation(FZ_REPLY_OK);
@@ -1033,8 +999,9 @@ int CSftpControlSocket::ListSubcommandResult(int prevResult)
 			pData->path.clear();
 			pData->subDir = _T("");
 			int res = ChangeDir();
-			if (res != FZ_REPLY_OK)
+			if (res != FZ_REPLY_OK) {
 				return res;
+			}
 		}
 		else {
 			ResetOperation(prevResult);
@@ -1046,7 +1013,7 @@ int CSftpControlSocket::ListSubcommandResult(int prevResult)
 		pData->path = m_CurrentPath;
 
 	if (!pData->refresh) {
-		wxASSERT(!pData->pNextOpData);
+		assert(!pData->pNextOpData);
 
 		// Do a cache lookup now that we know the correct directory
 
@@ -1101,7 +1068,7 @@ int CSftpControlSocket::ListSend()
 		// Check if we can use already existing listing
 		CDirectoryListing listing;
 		bool is_outdated = false;
-		wxASSERT(pData->subDir.empty()); // Did do ChangeDir before trying to lock
+		assert(pData->subDir.empty()); // Did do ChangeDir before trying to lock
 		bool found = engine_.GetDirectoryCache().Lookup(listing, *m_pCurrentServer, pData->path, true, is_outdated);
 		if (found && !is_outdated && !listing.get_unsure_flags() &&
 			listing.m_firstListTime >= pData->m_time_before_locking)
@@ -1118,18 +1085,9 @@ int CSftpControlSocket::ListSend()
 	}
 	else if (pData->opState == list_list) {
 		pData->pParser = new CDirectoryListingParser(this, *m_pCurrentServer, listingEncoding::unknown);
-		pData->pParser->SetTimezoneOffset(GetTimezoneOffset());
-		if (!SendCommand(_T("ls")))
-			return FZ_REPLY_ERROR;
-		return FZ_REPLY_WOULDBLOCK;
-	}
-	else if (pData->opState == list_mtime) {
-		LogMessage(MessageType::Status, _("Calculating timezone offset of server..."));
-		std::wstring const& name = pData->directoryListing[pData->mtime_index].name;
-		std::wstring quotedFilename = QuoteFilename(pData->directoryListing.path.FormatFilename(name, true).ToStdWstring());
-		if (!SendCommand(L"mtime " + WildcardEscape(quotedFilename),
-			L"mtime " + quotedFilename))
+		if (!SendCommand(_T("ls"))) {
 			return FZ_REPLY_ERROR;
+		}
 		return FZ_REPLY_WOULDBLOCK;
 	}
 
@@ -1150,30 +1108,34 @@ enum cwdStates
 	cwd_cwd_subdir
 };
 
-int CSftpControlSocket::ChangeDir(CServerPath path /*=CServerPath()*/, wxString subDir /*=_T("")*/, bool link_discovery /*=false*/)
+int CSftpControlSocket::ChangeDir(CServerPath path, std::wstring subDir, bool link_discovery)
 {
 	cwdStates state = cwd_init;
 
-	if (path.GetType() == DEFAULT)
+	if (path.GetType() == DEFAULT) {
 		path.SetType(m_pCurrentServer->GetType());
+	}
 
 	CServerPath target;
 	if (path.empty()) {
-		if (m_CurrentPath.empty())
+		if (m_CurrentPath.empty()) {
 			state = cwd_pwd;
-		else
+		}
+		else {
 			return FZ_REPLY_OK;
+		}
 	}
 	else {
 		if (!subDir.empty()) {
 			// Check if the target is in cache already
 			target = engine_.GetPathCache().Lookup(*m_pCurrentServer, path, subDir);
 			if (!target.empty()) {
-				if (m_CurrentPath == target)
+				if (m_CurrentPath == target) {
 					return FZ_REPLY_OK;
+				}
 
 				path = target;
-				subDir = _T("");
+				subDir.clear();
 				state = cwd_cwd;
 			}
 			else {
@@ -1183,14 +1145,16 @@ int CSftpControlSocket::ChangeDir(CServerPath path /*=CServerPath()*/, wxString
 					target.clear();
 					state = cwd_cwd_subdir;
 				}
-				else
+				else {
 					state = cwd_cwd;
+				}
 			}
 		}
 		else {
 			target = engine_.GetPathCache().Lookup(*m_pCurrentServer, path, _T(""));
-			if (m_CurrentPath == path || (!target.empty() && target == m_CurrentPath))
+			if (m_CurrentPath == path || (!target.empty() && target == m_CurrentPath)) {
 				return FZ_REPLY_OK;
+			}
 			state = cwd_cwd;
 		}
 	}
@@ -1207,7 +1171,7 @@ int CSftpControlSocket::ChangeDir(CServerPath path /*=CServerPath()*/, wxString
 		!static_cast<CSftpFileTransferOpData *>(pData->pNextOpData)->download)
 	{
 		pData->tryMkdOnFail = true;
-		wxASSERT(subDir.empty());
+		assert(subDir.empty());
 	}
 
 	m_pCurOpData = pData;
@@ -1215,10 +1179,9 @@ int CSftpControlSocket::ChangeDir(CServerPath path /*=CServerPath()*/, wxString
 	return SendNextCommand();
 }
 
-int CSftpControlSocket::ChangeDirParseResponse(bool successful, const wxString& reply)
+int CSftpControlSocket::ChangeDirParseResponse(bool successful, std::wstring const& reply)
 {
-	if (!m_pCurOpData)
-	{
+	if (!m_pCurOpData) {
 		ResetOperation(FZ_REPLY_ERROR);
 		return FZ_REPLY_ERROR;
 	}
@@ -1228,32 +1191,34 @@ int CSftpControlSocket::ChangeDirParseResponse(bool successful, const wxString&
 	switch (pData->opState)
 	{
 	case cwd_pwd:
-		if (!successful || reply.empty())
+		if (!successful || reply.empty()) {
 			error = true;
-		if (ParsePwdReply(reply))
-		{
+		}
+		if (ParsePwdReply(reply)) {
 			ResetOperation(FZ_REPLY_OK);
 			return FZ_REPLY_OK;
 		}
-		else
+		else {
 			error = true;
+		}
 		break;
 	case cwd_cwd:
-		if (!successful)
-		{
+		if (!successful) {
 			// Create remote directory if part of a file upload
-			if (pData->tryMkdOnFail)
-			{
+			if (pData->tryMkdOnFail) {
 				pData->tryMkdOnFail = false;
 				int res = Mkdir(pData->path);
-				if (res != FZ_REPLY_OK)
+				if (res != FZ_REPLY_OK) {
 					return res;
+				}
 			}
-			else
+			else {
 				error = true;
+			}
 		}
-		else if (reply.empty())
+		else if (reply.empty()) {
 			error = true;
+		}
 		else if (ParsePwdReply(reply)) {
 			engine_.GetPathCache().Store(*m_pCurrentServer, m_CurrentPath, pData->path);
 
@@ -1265,20 +1230,20 @@ int CSftpControlSocket::ChangeDirParseResponse(bool successful, const wxString&
 			pData->target.clear();
 			pData->opState = cwd_cwd_subdir;
 		}
-		else
+		else {
 			error = true;
+		}
 		break;
 	case cwd_cwd_subdir:
-		if (!successful || reply.empty())
-		{
-			if (pData->link_discovery)
-			{
+		if (!successful || reply.empty()) {
+			if (pData->link_discovery) {
 				LogMessage(MessageType::Debug_Info, _T("Symlink does not link to a directory, probably a file"));
 				ResetOperation(FZ_REPLY_LINKNOTDIR);
 				return FZ_REPLY_ERROR;
 			}
-			else
+			else {
 				error = true;
+			}
 		}
 		else if (ParsePwdReply(reply)) {
 			engine_.GetPathCache().Store(*m_pCurrentServer, m_CurrentPath, pData->path, pData->subDir);
@@ -1286,16 +1251,16 @@ int CSftpControlSocket::ChangeDirParseResponse(bool successful, const wxString&
 			ResetOperation(FZ_REPLY_OK);
 			return FZ_REPLY_OK;
 		}
-		else
+		else {
 			error = true;
+		}
 		break;
 	default:
 		error = true;
 		break;
 	}
 
-	if (error)
-	{
+	if (error) {
 		ResetOperation(FZ_REPLY_ERROR);
 		return FZ_REPLY_ERROR;
 	}
@@ -1303,9 +1268,9 @@ int CSftpControlSocket::ChangeDirParseResponse(bool successful, const wxString&
 	return SendNextCommand();
 }
 
-int CSftpControlSocket::ChangeDirSubcommandResult(int WXUNUSED(prevResult))
+int CSftpControlSocket::ChangeDirSubcommandResult(int prevResult)
 {
-	LogMessage(MessageType::Debug_Verbose, _T("CSftpControlSocket::ChangeDirSubcommandResult()"));
+	LogMessage(MessageType::Debug_Verbose, _T("CSftpControlSocket::ChangeDirSubcommandResult(%d)"), prevResult);
 
 	return SendNextCommand();
 }
@@ -1337,7 +1302,7 @@ int CSftpControlSocket::ChangeDirSend()
 				return FZ_REPLY_WOULDBLOCK;
 			}
 		}
-		cmd = L"cd " + QuoteFilename(pData->path.GetPath().ToStdWstring());
+		cmd = L"cd " + QuoteFilename(pData->path.GetPath());
 		m_CurrentPath.clear();
 		break;
 	case cwd_cwd_subdir:
@@ -1346,7 +1311,7 @@ int CSftpControlSocket::ChangeDirSend()
 			return FZ_REPLY_ERROR;
 		}
 		else {
-			cmd = L"cd " + QuoteFilename(pData->subDir.ToStdWstring());
+			cmd = L"cd " + QuoteFilename(pData->subDir);
 		}
 		m_CurrentPath.clear();
 		break;
@@ -1394,7 +1359,7 @@ void CSftpControlSocket::ProcessReply(int result, std::wstring const& reply)
 		RenameParseResponse(result == FZ_REPLY_OK, reply);
 		break;
 	default:
-		LogMessage(MessageType::Debug_Warning, _T("No action for parsing replies to command %d"), (int)commandId);
+		LogMessage(MessageType::Debug_Warning, _T("No action for parsing replies to command %d"), commandId);
 		ResetOperation(FZ_REPLY_INTERNALERROR);
 		break;
 	}
@@ -1404,19 +1369,20 @@ int CSftpControlSocket::ResetOperation(int nErrorCode)
 {
 	LogMessage(MessageType::Debug_Verbose, _T("CSftpControlSocket::ResetOperation(%d)"), nErrorCode);
 
-	if (m_pCurOpData && m_pCurOpData->opId == Command::connect)
-	{
+	if (m_pCurOpData && m_pCurOpData->opId == Command::connect) {
 		CSftpConnectOpData *pData = static_cast<CSftpConnectOpData *>(m_pCurOpData);
-		if (pData->opState == connect_init && (nErrorCode & FZ_REPLY_CANCELED) != FZ_REPLY_CANCELED)
+		if (pData->opState == connect_init && (nErrorCode & FZ_REPLY_CANCELED) != FZ_REPLY_CANCELED) {
 			LogMessage(MessageType::Error, _("fzsftp could not be started"));
-		if (pData->criticalFailure)
+		}
+		if (pData->criticalFailure) {
 			nErrorCode |= FZ_REPLY_CRITICALERROR;
+		}
 	}
-	if (m_pCurOpData && m_pCurOpData->opId == Command::del && !(nErrorCode & FZ_REPLY_DISCONNECTED))
-	{
+	if (m_pCurOpData && m_pCurOpData->opId == Command::del && !(nErrorCode & FZ_REPLY_DISCONNECTED)) {
 		CSftpDeleteOpData *pData = static_cast<CSftpDeleteOpData *>(m_pCurOpData);
-		if (pData->m_needSendListing)
+		if (pData->m_needSendListing) {
 			engine_.SendDirectoryListingNotification(pData->path, false, true, false);
+		}
 	}
 
 	return CControlSocket::ResetOperation(nErrorCode);
@@ -1472,15 +1438,17 @@ int CSftpControlSocket::FileTransfer(std::wstring const& localFile, CServerPath
 	LogMessage(MessageType::Debug_Verbose, _T("CSftpControlSocket::FileTransfer(...)"));
 
 	if (localFile.empty()) {
-		if (!download)
+		if (!download) {
 			ResetOperation(FZ_REPLY_CRITICALERROR | FZ_REPLY_NOTSUPPORTED);
-		else
+		}
+		else {
 			ResetOperation(FZ_REPLY_SYNTAXERROR);
+		}
 		return FZ_REPLY_ERROR;
 	}
 
 	if (download) {
-		wxString filename = remotePath.FormatFilename(remoteFile);
+		std::wstring filename = remotePath.FormatFilename(remoteFile);
 		LogMessage(MessageType::Status, _("Starting download of %s"), filename);
 	}
 	else {
@@ -1658,7 +1626,7 @@ int CSftpControlSocket::FileTransferSend()
 
 			engine_.transfer_status_.Init(pData->remoteFileSize, pData->resume ? pData->localFileSize : 0, false);
 			cmd += L"get ";
-			cmd += QuoteFilename(pData->remotePath.FormatFilename(pData->remoteFile, !pData->tryAbsolutePath).ToStdWstring()) + L" ";
+			cmd += QuoteFilename(pData->remotePath.FormatFilename(pData->remoteFile, !pData->tryAbsolutePath)) + L" ";
 
 			std::wstring localFile = QuoteFilename(pData->localFile);
 			std::wstring logstr = cmd;
@@ -1676,7 +1644,7 @@ int CSftpControlSocket::FileTransferSend()
 
 			std::wstring logstr = cmd;
 			std::wstring localFile = QuoteFilename(pData->localFile) + L" ";
-			std::wstring remoteFile = QuoteFilename(pData->remotePath.FormatFilename(pData->remoteFile, !pData->tryAbsolutePath).ToStdWstring());
+			std::wstring remoteFile = QuoteFilename(pData->remotePath.FormatFilename(pData->remoteFile, !pData->tryAbsolutePath));
 
 			logstr += localFile;
 			logstr += remoteFile;
@@ -1694,20 +1662,20 @@ int CSftpControlSocket::FileTransferSend()
 		pData->transferInitiated = true;
 	}
 	else if (pData->opState == filetransfer_mtime) {
-		std::wstring quotedFilename = QuoteFilename(pData->remotePath.FormatFilename(pData->remoteFile, !pData->tryAbsolutePath).ToStdWstring());
+		std::wstring quotedFilename = QuoteFilename(pData->remotePath.FormatFilename(pData->remoteFile, !pData->tryAbsolutePath));
 		if (!SendCommand(_T("mtime ") + WildcardEscape(quotedFilename),
 			L"mtime " + quotedFilename))
 			return FZ_REPLY_ERROR;
 	}
 	else if (pData->opState == filetransfer_chmtime) {
-		wxASSERT(!pData->fileTime.empty());
+		assert(!pData->fileTime.empty());
 		if (pData->download) {
 			LogMessage(__TFILE__, __LINE__, this, MessageType::Debug_Info, _T("  filetransfer_chmtime during download"));
 			ResetOperation(FZ_REPLY_INTERNALERROR);
 			return FZ_REPLY_ERROR;
 		}
 
-		std::wstring quotedFilename = QuoteFilename(pData->remotePath.FormatFilename(pData->remoteFile, !pData->tryAbsolutePath).ToStdWstring());
+		std::wstring quotedFilename = QuoteFilename(pData->remotePath.FormatFilename(pData->remoteFile, !pData->tryAbsolutePath));
 
 		fz::datetime t = pData->fileTime;
 		t -= fz::duration::from_minutes(m_pCurrentServer->GetTimezoneOffset());
@@ -1723,7 +1691,7 @@ int CSftpControlSocket::FileTransferSend()
 	return FZ_REPLY_WOULDBLOCK;
 }
 
-int CSftpControlSocket::FileTransferParseResponse(int result, const wxString& reply)
+int CSftpControlSocket::FileTransferParseResponse(int result, std::wstring const& reply)
 {
 	LogMessage(MessageType::Debug_Verbose, _T("FileTransferParseResponse(%d)"), result);
 
@@ -1761,8 +1729,7 @@ int CSftpControlSocket::FileTransferParseResponse(int result, const wxString& re
 		if (result == FZ_REPLY_OK && !reply.empty()) {
 			time_t seconds = 0;
 			bool parsed = true;
-			for (unsigned int i = 0; i < reply.Len(); ++i) {
-				wxChar c = reply[i];
+			for (auto c : reply) {
 				if (c < '0' || c > '9') {
 					parsed = false;
 					break;
@@ -1780,8 +1747,9 @@ int CSftpControlSocket::FileTransferParseResponse(int result, const wxString& re
 		}
 		pData->opState = filetransfer_transfer;
 		int res = CheckOverwriteFile();
-		if (res != FZ_REPLY_OK)
+		if (res != FZ_REPLY_OK) {
 			return res;
+		}
 
 		return SendNextCommand();
 	}
@@ -1802,7 +1770,7 @@ int CSftpControlSocket::FileTransferParseResponse(int result, const wxString& re
 	return FZ_REPLY_OK;
 }
 
-int CSftpControlSocket::DoClose(int nErrorCode /*=FZ_REPLY_DISCONNECTED*/)
+int CSftpControlSocket::DoClose(int nErrorCode)
 {
 	engine_.GetRateLimiter().RemoveObject(this);
 
@@ -1811,7 +1779,6 @@ int CSftpControlSocket::DoClose(int nErrorCode /*=FZ_REPLY_DISCONNECTED*/)
 	}
 
 	if (m_pInputThread) {
-		m_pInputThread->join();
 		delete m_pInputThread;
 		m_pInputThread = 0;
 
@@ -1894,7 +1861,7 @@ int CSftpControlSocket::Mkdir(const CServerPath& path)
 	return SendNextCommand();
 }
 
-int CSftpControlSocket::MkdirParseResponse(bool successful, const wxString&)
+int CSftpControlSocket::MkdirParseResponse(bool successful, std::wstring const&)
 {
 	LogMessage(MessageType::Debug_Verbose, _T("CSftpControlSocket::MkdirParseResonse"));
 
@@ -2003,13 +1970,13 @@ int CSftpControlSocket::MkdirSend()
 	case mkd_findparent:
 	case mkd_cwdsub:
 		m_CurrentPath.clear();
-		res = SendCommand(L"cd " + QuoteFilename(pData->currentPath.GetPath().ToStdWstring()));
+		res = SendCommand(L"cd " + QuoteFilename(pData->currentPath.GetPath()));
 		break;
 	case mkd_mkdsub:
-		res = SendCommand(L"mkdir " + QuoteFilename(pData->segments.back().ToStdWstring()));
+		res = SendCommand(L"mkdir " + QuoteFilename(pData->segments.back()));
 		break;
 	case mkd_tryfull:
-		res = SendCommand(L"mkdir " + QuoteFilename(pData->path.GetPath().ToStdWstring()));
+		res = SendCommand(L"mkdir " + QuoteFilename(pData->path.GetPath()));
 		break;
 	default:
 		LogMessage(__TFILE__, __LINE__, this, MessageType::Debug_Warning, _T("unknown op state: %d"), pData->opState);
@@ -2032,19 +1999,19 @@ std::wstring CSftpControlSocket::QuoteFilename(std::wstring const& filename)
 int CSftpControlSocket::Delete(const CServerPath& path, std::deque<std::wstring>&& files)
 {
 	LogMessage(MessageType::Debug_Verbose, _T("CSftpControlSocket::Delete"));
-	wxASSERT(!m_pCurOpData);
+	assert(!m_pCurOpData);
 	CSftpDeleteOpData *pData = new CSftpDeleteOpData();
 	m_pCurOpData = pData;
 	pData->path = path;
 	pData->files = files;
 
 	// CFileZillaEnginePrivate should have checked this already
-	wxASSERT(!files.empty());
+	assert(!files.empty());
 
 	return SendNextCommand();
 }
 
-int CSftpControlSocket::DeleteParseResponse(bool successful, const wxString&)
+int CSftpControlSocket::DeleteParseResponse(bool successful, std::wstring const&)
 {
 	LogMessage(MessageType::Debug_Verbose, _T("CSftpControlSocket::DeleteParseResponse"));
 
@@ -2060,7 +2027,7 @@ int CSftpControlSocket::DeleteParseResponse(bool successful, const wxString&)
 		pData->m_deleteFailed = true;
 	}
 	else {
-		const wxString& file = pData->files.front();
+		std::wstring const& file = pData->files.front();
 
 		engine_.GetDirectoryCache().RemoveFile(*m_pCurrentServer, pData->path, file);
 
@@ -2093,14 +2060,14 @@ int CSftpControlSocket::DeleteSend()
 	}
 	CSftpDeleteOpData *pData = static_cast<CSftpDeleteOpData *>(m_pCurOpData);
 
-	const wxString& file = pData->files.front();
+	std::wstring const& file = pData->files.front();
 	if (file.empty()) {
 		LogMessage(__TFILE__, __LINE__, this, MessageType::Debug_Info, _T("Empty filename"));
 		ResetOperation(FZ_REPLY_INTERNALERROR);
 		return FZ_REPLY_ERROR;
 	}
 
-	std::wstring filename = pData->path.FormatFilename(file).ToStdWstring();
+	std::wstring filename = pData->path.FormatFilename(file);
 	if (filename.empty()) {
 		LogMessage(MessageType::Error, _("Filename cannot be constructed for directory %s and filename %s"), pData->path.GetPath(), file);
 		ResetOperation(FZ_REPLY_ERROR);
@@ -2115,7 +2082,9 @@ int CSftpControlSocket::DeleteSend()
 
 	if (!SendCommand(L"rm " + WildcardEscape(QuoteFilename(filename)),
 			  L"rm " + QuoteFilename(filename)))
+	{
 		return FZ_REPLY_ERROR;
+	}
 
 	return FZ_REPLY_WOULDBLOCK;
 }
@@ -2131,14 +2100,14 @@ public:
 	virtual ~CSftpRemoveDirOpData() {}
 
 	CServerPath path;
-	wxString subDir;
+	std::wstring subDir;
 };
 
-int CSftpControlSocket::RemoveDir(const CServerPath& path /*=CServerPath()*/, const wxString& subDir /*=_T("")*/)
+int CSftpControlSocket::RemoveDir(CServerPath const& path, std::wstring const& subDir)
 {
 	LogMessage(MessageType::Debug_Verbose, _T("CSftpControlSocket::RemoveDir"));
 
-	wxASSERT(!m_pCurOpData);
+	assert(!m_pCurOpData);
 	CSftpRemoveDirOpData *pData = new CSftpRemoveDirOpData();
 	m_pCurOpData = pData;
 	pData->path = path;
@@ -2159,7 +2128,7 @@ int CSftpControlSocket::RemoveDir(const CServerPath& path /*=CServerPath()*/, co
 	engine_.GetPathCache().InvalidatePath(*m_pCurrentServer, pData->path, pData->subDir);
 
 	engine_.InvalidateCurrentWorkingDirs(fullPath);
-	std::wstring quotedFilename = QuoteFilename(fullPath.GetPath().ToStdWstring());
+	std::wstring quotedFilename = QuoteFilename(fullPath.GetPath());
 	if (!SendCommand(L"rmdir " + WildcardEscape(quotedFilename),
 			  L"rmdir " + quotedFilename))
 		return FZ_REPLY_ERROR;
@@ -2167,7 +2136,7 @@ int CSftpControlSocket::RemoveDir(const CServerPath& path /*=CServerPath()*/, co
 	return FZ_REPLY_WOULDBLOCK;
 }
 
-int CSftpControlSocket::RemoveDirParseResponse(bool successful, const wxString&)
+int CSftpControlSocket::RemoveDirParseResponse(bool successful, std::wstring const&)
 {
 	LogMessage(MessageType::Debug_Verbose, _T("CSftpControlSocket::RemoveDirParseResponse"));
 
@@ -2239,7 +2208,7 @@ int CSftpControlSocket::Chmod(const CChmodCommand& command)
 	return SendNextCommand();
 }
 
-int CSftpControlSocket::ChmodParseResponse(bool successful, const wxString&)
+int CSftpControlSocket::ChmodParseResponse(bool successful, std::wstring const&)
 {
 	CSftpChmodOpData *pData = static_cast<CSftpChmodOpData*>(m_pCurOpData);
 	if (!pData)
@@ -2295,10 +2264,10 @@ int CSftpControlSocket::ChmodSend()
 		{
 			engine_.GetDirectoryCache().UpdateFile(*m_pCurrentServer, pData->m_cmd.GetPath(), pData->m_cmd.GetFile(), false, CDirectoryCache::unknown);
 
-			std::wstring quotedFilename = QuoteFilename(pData->m_cmd.GetPath().FormatFilename(pData->m_cmd.GetFile(), !pData->m_useAbsolute).ToStdWstring());
+			std::wstring quotedFilename = QuoteFilename(pData->m_cmd.GetPath().FormatFilename(pData->m_cmd.GetFile(), !pData->m_useAbsolute));
 
-			res = SendCommand(L"chmod " + pData->m_cmd.GetPermission().ToStdWstring() + L" " + WildcardEscape(quotedFilename),
-					   L"chmod " + pData->m_cmd.GetPermission().ToStdWstring() + _T(" ") + quotedFilename);
+			res = SendCommand(L"chmod " + pData->m_cmd.GetPermission() + L" " + WildcardEscape(quotedFilename),
+					   L"chmod " + pData->m_cmd.GetPermission() + L" " + quotedFilename);
 		}
 		break;
 	default:
@@ -2338,8 +2307,7 @@ enum renameStates
 
 int CSftpControlSocket::Rename(const CRenameCommand& command)
 {
-	if (m_pCurOpData)
-	{
+	if (m_pCurOpData) {
 		LogMessage(__TFILE__, __LINE__, this, MessageType::Debug_Warning, _T("m_pCurOpData not empty"));
 		ResetOperation(FZ_REPLY_INTERNALERROR);
 		return FZ_REPLY_ERROR;
@@ -2352,24 +2320,23 @@ int CSftpControlSocket::Rename(const CRenameCommand& command)
 	m_pCurOpData = pData;
 
 	int res = ChangeDir(command.GetFromPath());
-	if (res != FZ_REPLY_OK)
+	if (res != FZ_REPLY_OK) {
 		return res;
+	}
 
 	return SendNextCommand();
 }
 
-int CSftpControlSocket::RenameParseResponse(bool successful, const wxString&)
+int CSftpControlSocket::RenameParseResponse(bool successful, std::wstring const&)
 {
 	CSftpRenameOpData *pData = static_cast<CSftpRenameOpData*>(m_pCurOpData);
-	if (!pData)
-	{
+	if (!pData) {
 		LogMessage(__TFILE__, __LINE__, this, MessageType::Debug_Warning, _T("m_pCurOpData empty"));
 		ResetOperation(FZ_REPLY_INTERNALERROR);
 		return FZ_REPLY_ERROR;
 	}
 
-	if (!successful)
-	{
+	if (!successful) {
 		ResetOperation(FZ_REPLY_ERROR);
 		return FZ_REPLY_ERROR;
 	}
@@ -2426,8 +2393,8 @@ int CSftpControlSocket::RenameSend()
 			engine_.GetDirectoryCache().InvalidateFile(*m_pCurrentServer, pData->m_cmd.GetFromPath(), pData->m_cmd.GetFromFile(), &wasDir);
 			engine_.GetDirectoryCache().InvalidateFile(*m_pCurrentServer, pData->m_cmd.GetToPath(), pData->m_cmd.GetToFile());
 
-			std::wstring fromQuoted = QuoteFilename(pData->m_cmd.GetFromPath().FormatFilename(pData->m_cmd.GetFromFile(), !pData->m_useAbsolute).ToStdWstring());
-			std::wstring toQuoted = QuoteFilename(pData->m_cmd.GetToPath().FormatFilename(pData->m_cmd.GetToFile(), !pData->m_useAbsolute && pData->m_cmd.GetFromPath() == pData->m_cmd.GetToPath()).ToStdWstring());
+			std::wstring fromQuoted = QuoteFilename(pData->m_cmd.GetFromPath().FormatFilename(pData->m_cmd.GetFromFile(), !pData->m_useAbsolute));
+			std::wstring toQuoted = QuoteFilename(pData->m_cmd.GetToPath().FormatFilename(pData->m_cmd.GetToFile(), !pData->m_useAbsolute && pData->m_cmd.GetFromPath() == pData->m_cmd.GetToPath()));
 
 			engine_.GetPathCache().InvalidatePath(*m_pCurrentServer, pData->m_cmd.GetFromPath(), pData->m_cmd.GetFromFile());
 			engine_.GetPathCache().InvalidatePath(*m_pCurrentServer, pData->m_cmd.GetToPath(), pData->m_cmd.GetToFile());
@@ -2495,25 +2462,28 @@ void CSftpControlSocket::OnQuotaRequest(CRateLimiter::rate_direction direction)
 	int64_t bytes = GetAvailableBytes(direction);
 	if (bytes > 0) {
 		int b;
-		if (bytes > INT_MAX)
+		if (bytes > INT_MAX) {
 			b = INT_MAX;
-		else
+		}
+		else {
 			b = bytes;
-		AddToStream(wxString::Format(_T("-%d%d,%d\n"), (int)direction, b, engine_.GetOptions().GetOptionVal(OPTION_SPEEDLIMIT_INBOUND + static_cast<int>(direction))));
+		}
+		AddToStream(fz::sprintf(L"-%d%d,%d\n", direction, b, engine_.GetOptions().GetOptionVal(OPTION_SPEEDLIMIT_INBOUND + static_cast<int>(direction))));
 		UpdateUsage(direction, b);
 	}
-	else if (bytes == 0)
+	else if (bytes == 0) {
 		Wait(direction);
-	else if (bytes < 0)
-		AddToStream(wxString::Format(_T("-%d-\n"), (int)direction));
+	}
+	else if (bytes < 0) {
+		AddToStream(fz::sprintf(L"-%d-\n", direction));
+	}
 }
 
 
 int CSftpControlSocket::ParseSubcommandResult(int prevResult)
 {
 	LogMessage(MessageType::Debug_Verbose, _T("CSftpControlSocket::ParseSubcommandResult(%d)"), prevResult);
-	if (!m_pCurOpData)
-	{
+	if (!m_pCurOpData) {
 		LogMessage(__TFILE__, __LINE__, this, MessageType::Debug_Warning, _T("ParseSubcommandResult called without active operation"));
 		ResetOperation(FZ_REPLY_ERROR);
 		return FZ_REPLY_ERROR;
@@ -2540,30 +2510,6 @@ int CSftpControlSocket::ParseSubcommandResult(int prevResult)
 	return FZ_REPLY_ERROR;
 }
 
-int CSftpControlSocket::ListCheckTimezoneDetection()
-{
-	wxASSERT(m_pCurOpData);
-
-	CSftpListOpData *pData = static_cast<CSftpListOpData *>(m_pCurOpData);
-
-	if (CServerCapabilities::GetCapability(*m_pCurrentServer, timezone_offset) == unknown) {
-		const int count = pData->directoryListing.GetCount();
-		for (int i = 0; i < count; ++i) {
-			if (!pData->directoryListing[i].has_time())
-				continue;
-
-			if (pData->directoryListing[i].is_link())
-				continue;
-
-			pData->opState = list_mtime;
-			pData->mtime_index = i;
-			return SendNextCommand();
-		}
-	}
-
-	return FZ_REPLY_OK;
-}
-
 void CSftpControlSocket::operator()(fz::event_base const& ev)
 {
 	if (fz::dispatch<CSftpEvent, CTerminateEvent>(ev, this,
