@@ -9,6 +9,8 @@
 
 #include <algorithm>
 
+#include <string.h>
+
 #if FZ_USE_GNUTLS_SYSTEM_CIPHERS
 char const ciphers[] = "@SYSTEM";
 #else
@@ -26,7 +28,7 @@ void log_func(int level, const char* msg)
 	}
 	std::wstring s = fz::to_wstring(msg);
 	fz::trim(s);
-	pLoggingControlSocket->LogMessage(MessageType::Debug_Debug, _T("tls: %d %s"), level, s);
+	pLoggingControlSocket->LogMessage(MessageType::Debug_Debug, L"tls: %d %s", level, s);
 }
 #endif
 
@@ -616,8 +618,8 @@ int CTlsSocket::Handshake(const CTlsSocket* pPrimarySocket, bool try_resume)
 int CTlsSocket::ContinueHandshake()
 {
 	m_pOwner->LogMessage(MessageType::Debug_Verbose, _T("CTlsSocket::ContinueHandshake()"));
-	wxASSERT(m_session);
-	wxASSERT(m_tlsState == TlsState::handshake);
+	assert(m_session);
+	assert(m_tlsState == TlsState::handshake);
 
 	int res = gnutls_handshake(m_session);
 	while (res == GNUTLS_E_AGAIN || res == GNUTLS_E_INTERRUPTED) {
@@ -821,7 +823,7 @@ void CTlsSocket::CheckResumeFailedReadWrite()
 		m_canTriggerWrite = true;
 	}
 	if (m_lastReadFailed) {
-		wxASSERT(!m_peekData);
+		assert(!m_peekData);
 
 		m_peekDataLen = 65536;
 		m_peekData = new char[m_peekDataLen];
@@ -1090,12 +1092,14 @@ bool CTlsSocket::ExtractCert(gnutls_x509_crt_t const& cert, CCertificate& out)
 	}
 
 	datum_holder der;
-	if (gnutls_x509_crt_export2(cert, GNUTLS_X509_FMT_DER, &der) != GNUTLS_E_SUCCESS) {
+	if (gnutls_x509_crt_export2(cert, GNUTLS_X509_FMT_DER, &der) != GNUTLS_E_SUCCESS || !der.data || !der.size) {
 		m_pOwner->LogMessage(MessageType::Error, _T("gnutls_x509_crt_get_issuer_dn"));
 		return false;
 	}
+	std::vector<uint8_t> data(der.data, der.data + der.size);
+
 	out = CCertificate(
-		der.data, der.size,
+		std::move(data),
 		activationTime, expirationTime,
 		serial,
 		pkAlgoName, pkBits,
@@ -1104,7 +1108,7 @@ bool CTlsSocket::ExtractCert(gnutls_x509_crt_t const& cert, CCertificate& out)
 		fingerprint_sha1,
 		issuer,
 		subject,
-		alt_subject_names);
+		std::move(alt_subject_names));
 
 	return true;
 }
@@ -1416,7 +1420,7 @@ std::wstring CTlsSocket::GetProtocolName()
 	}
 	
 	if (ret.empty()) {
-		ret = _("unknown").ToStdWstring();
+		ret = _("unknown");
 	}
 
 	return ret;
@@ -1432,7 +1436,7 @@ std::wstring CTlsSocket::GetKeyExchange()
 	}
 
 	if (ret.empty()) {
-		ret = _("unknown").ToStdWstring();
+		ret = _("unknown");
 	}
 
 	return ret;
@@ -1448,7 +1452,7 @@ std::wstring CTlsSocket::GetCipherName()
 	}
 
 	if (ret.empty()) {
-		ret = _("unknown").ToStdWstring();
+		ret = _("unknown");
 	}
 
 	return ret;
@@ -1464,7 +1468,7 @@ std::wstring CTlsSocket::GetMacName()
 	}
 	
 	if (ret.empty()) {
-		ret = _("unknown").ToStdWstring();
+		ret = _("unknown");
 	}
 
 	return ret;
