@@ -127,10 +127,12 @@ void CTransferSocket::OnSocketEvent(CSocketEventSource*, SocketEventType t, int
 	}
 
 	if (m_pSocketServer) {
-		if (t == SocketEventType::connection)
+		if (t == SocketEventType::connection) {
 			OnAccept(error);
-		else
+		}
+		else {
 			controlSocket_.LogMessage(MessageType::Debug_Info, _T("Unhandled socket event %d from listening socket"), t);
+		}
 		return;
 	}
 
@@ -173,8 +175,9 @@ void CTransferSocket::OnAccept(int error)
 
 	m_pSocket = m_pSocketServer->Accept(error);
 	if (!m_pSocket) {
-		if (error == EAGAIN)
+		if (error == EAGAIN) {
 			controlSocket_.LogMessage(MessageType::Debug_Verbose, _T("No pending connection"));
+		}
 		else {
 			controlSocket_.LogMessage(MessageType::Status, _("Could not accept connection: %s"), CSocket::GetErrorDescription(error));
 			TransferEnd(TransferEndReason::transfer_failure);
@@ -211,8 +214,9 @@ void CTransferSocket::OnConnect()
 		}
 	}
 
-	if (m_bActive)
+	if (m_bActive) {
 		TriggerPostponedEvents();
+	}
 }
 
 void CTransferSocket::OnReceive()
@@ -242,14 +246,14 @@ void CTransferSocket::OnReceive()
 					controlSocket_.LogMessage(MessageType::Error, _T("Could not read from transfer socket: %s"), CSocket::GetErrorDescription(error));
 					TransferEnd(TransferEndReason::transfer_failure);
 				}
-				else if (m_onCloseCalled && !m_pBackend->IsWaiting(CRateLimiter::inbound))
+				else if (m_onCloseCalled && !m_pBackend->IsWaiting(CRateLimiter::inbound)) {
 					TransferEnd(TransferEndReason::successful);
+				}
 				return;
 			}
 
 			if (numread > 0) {
-				if (!m_pDirectoryListingParser->AddData(pBuffer, numread))
-				{
+				if (!m_pDirectoryListingParser->AddData(pBuffer, numread)) {
 					TransferEnd(TransferEndReason::transfer_failure);
 					return;
 				}
@@ -276,8 +280,9 @@ void CTransferSocket::OnReceive()
 		// Otherwise this behaves like a livelock on very large files written to a very fast
 		// SSD downloaded from a very fast server.
 		for (int i = 0; i < 100; ++i) {
-			if (!CheckGetNextWriteBuffer())
+			if (!CheckGetNextWriteBuffer()) {
 				return;
+			}
 
 			numread = m_pBackend->Read(m_pTransferBuffer, m_transferBufferLen, error);
 			if (numread <= 0) {
@@ -322,10 +327,10 @@ void CTransferSocket::OnReceive()
 					TransferEnd(TransferEndReason::transfer_failure);
 				}
 				else if (m_onCloseCalled && !m_pBackend->IsWaiting(CRateLimiter::inbound)) {
-					if (m_transferBufferLen == 1)
+					if (m_transferBufferLen == 1) {
 						TransferEnd(TransferEndReason::successful);
-					else
-					{
+					}
+					else {
 						controlSocket_.LogMessage(MessageType::Debug_Warning, _T("Server incorrectly sent %d bytes"), m_transferBufferLen);
 						TransferEnd(TransferEndReason::failed_resumetest);
 					}
@@ -366,8 +371,9 @@ void CTransferSocket::OnSend()
 		return;
 	}
 
-	if (m_transferMode != TransferMode::upload)
+	if (m_transferMode != TransferMode::upload) {
 		return;
+	}
 
 	int error;
 	int written;
@@ -376,12 +382,14 @@ void CTransferSocket::OnSend()
 	// Otherwise this behaves like a livelock on very large files read from a very fast
 	// SSD uploaded to a very fast server.
 	for (int i = 0; i < 100; ++i) {
-		if (!CheckGetNextReadBuffer())
+		if (!CheckGetNextReadBuffer()) {
 			return;
+		}
 
 		written = m_pBackend->Write(m_pTransferBuffer, m_transferBufferLen, error);
-		if (written <= 0)
+		if (written <= 0) {
 			break;
+		}
 
 		controlSocket_.SetActive(CFileZillaEngine::send);
 		if (m_madeProgress == 1) {
@@ -482,8 +490,10 @@ void CTransferSocket::OnClose(int error)
 	}
 }
 
-bool CTransferSocket::SetupPassiveTransfer(wxString host, int port)
+bool CTransferSocket::SetupPassiveTransfer(std::wstring const& host, int port)
 {
+	std::string ip;
+
 	ResetSocket();
 
 	m_pSocket = new CSocket(engine_.GetThreadPool(), this);
@@ -492,7 +502,7 @@ bool CTransferSocket::SetupPassiveTransfer(wxString host, int port)
 		m_pProxyBackend = new CProxySocket(this, m_pSocket, &controlSocket_);
 
 		int res = m_pProxyBackend->Handshake(controlSocket_.m_pProxyBackend->GetProxyType(),
-											 host.ToStdWstring(), port,
+											 host, port,
 											 controlSocket_.m_pProxyBackend->GetUser(), controlSocket_.m_pProxyBackend->GetPass());
 
 		if (res != EINPROGRESS) {
@@ -500,14 +510,17 @@ bool CTransferSocket::SetupPassiveTransfer(wxString host, int port)
 			return false;
 		}
 		int error;
-		host = controlSocket_.m_pSocket->GetPeerIP();
+		ip = controlSocket_.m_pSocket->GetPeerIP();
 		port = controlSocket_.m_pSocket->GetRemotePort(error);
-		if (host.empty() || port < 1) {
+		if (ip.empty() || port < 1) {
 			controlSocket_.LogMessage(MessageType::Debug_Warning, _T("Could not get peer address of control connection."));
 			ResetSocket();
 			return false;
 		}
 	}
+	else {
+		ip = fz::to_utf8(host);
+	}
 
 	SetSocketBufferSizes(m_pSocket);
 
@@ -525,7 +538,7 @@ bool CTransferSocket::SetupPassiveTransfer(wxString host, int port)
 		controlSocket_.LogMessage(MessageType::Debug_Info, _T("Binding data connection source IP to control connection source IP %s"), bindAddress);
 	}
 	else {
-		if (controlSocket_.m_pSocket->GetPeerIP(true) == host || controlSocket_.m_pSocket->GetPeerIP(false) == host) {
+		if (controlSocket_.m_pSocket->GetPeerIP(true) == ip || controlSocket_.m_pSocket->GetPeerIP(false) == ip) {
 			bindAddress = controlSocket_.m_pSocket->GetLocalIP();
 			controlSocket_.LogMessage(MessageType::Debug_Info, _T("Binding data connection source IP to control connection source IP %s"), bindAddress);
 		}
@@ -534,7 +547,7 @@ bool CTransferSocket::SetupPassiveTransfer(wxString host, int port)
 		}
 	}
 
-	int res = m_pSocket->Connect(fz::to_native(host), port, CSocket::unspec, bindAddress);
+	int res = m_pSocket->Connect(fz::to_native(ip), port, CSocket::unspec, bindAddress);
 	if (res && res != EINPROGRESS) {
 		ResetSocket();
 		return false;
@@ -545,8 +558,9 @@ bool CTransferSocket::SetupPassiveTransfer(wxString host, int port)
 
 void CTransferSocket::SetActive()
 {
-	if (m_transferEndReason != TransferEndReason::none)
+	if (m_transferEndReason != TransferEndReason::none) {
 		return;
+	}
 	if (m_transferMode == TransferMode::download || m_transferMode == TransferMode::upload) {
 		if (ioThread_) {
 			ioThread_->SetEventHandler(this);
@@ -554,11 +568,13 @@ void CTransferSocket::SetActive()
 	}
 
 	m_bActive = true;
-	if (!m_pSocket)
+	if (!m_pSocket) {
 		return;
+	}
 
-	if (m_pSocket->GetState() == CSocket::connected || m_pSocket->GetState() == CSocket::closing)
+	if (m_pSocket->GetState() == CSocket::connected || m_pSocket->GetState() == CSocket::closing) {
 		TriggerPostponedEvents();
+	}
 }
 
 void CTransferSocket::TransferEnd(TransferEndReason reason)
@@ -611,12 +627,13 @@ CSocket* CTransferSocket::CreateSocketServer()
 
 	int low = engine_.GetOptions().GetOptionVal(OPTION_LIMITPORTS_LOW);
 	int high = engine_.GetOptions().GetOptionVal(OPTION_LIMITPORTS_HIGH);
-	if (low > high)
+	if (low > high) {
 		low = high;
+	}
 
 	if (start < low || start > high) {
 		start = fz::random_number(low, high);
-		wxASSERT(start >= low && start <= high);
+		assert(start >= low && start <= high);
 	}
 
 	CSocket* pServer = 0;
@@ -624,10 +641,12 @@ CSocket* CTransferSocket::CreateSocketServer()
 	int count = high - low + 1;
 	while (count--) {
 		pServer = CreateSocketServer(start++);
-		if (pServer)
+		if (pServer) {
 			break;
-		if (start > high)
+		}
+		if (start > high) {
 			start = low;
+		}
 	}
 
 	return pServer;
@@ -638,14 +657,17 @@ bool CTransferSocket::CheckGetNextWriteBuffer()
 	if (!m_transferBufferLen) {
 		int res = ioThread_->GetNextWriteBuffer(&m_pTransferBuffer);
 
-		if (res == IO_Again)
+		if (res == IO_Again) {
 			return false;
+		}
 		else if (res == IO_Error) {
-			wxString error = ioThread_->GetError();
-			if (error.empty() )
+			std::wstring error = ioThread_->GetError();
+			if (error.empty()) {
 				controlSocket_.LogMessage(MessageType::Error, _("Can't write data to file."));
-			else
+			}
+			else {
 				controlSocket_.LogMessage(MessageType::Error, _("Can't write data to file: %s"), error);
+			}
 			TransferEnd(TransferEndReason::transfer_failure_critical);
 			return false;
 		}
@@ -689,13 +711,16 @@ bool CTransferSocket::CheckGetNextReadBuffer()
 
 void CTransferSocket::OnIOThreadEvent()
 {
-	if (!m_bActive || m_transferEndReason != TransferEndReason::none)
+	if (!m_bActive || m_transferEndReason != TransferEndReason::none) {
 		return;
+	}
 
-	if (m_transferMode == TransferMode::download)
+	if (m_transferMode == TransferMode::download) {
 		OnReceive();
-	else if (m_transferMode == TransferMode::upload)
+	}
+	else if (m_transferMode == TransferMode::upload) {
 		OnSend();
+	}
 }
 
 void CTransferSocket::FinalizeWrite()
@@ -703,17 +728,21 @@ void CTransferSocket::FinalizeWrite()
 	bool res = ioThread_->Finalize(BUFFERSIZE - m_transferBufferLen);
 	m_transferBufferLen = BUFFERSIZE;
 
-	if (m_transferEndReason != TransferEndReason::none)
+	if (m_transferEndReason != TransferEndReason::none) {
 		return;
+	}
 
-	if (res)
+	if (res) {
 		TransferEnd(TransferEndReason::successful);
+	}
 	else {
-		wxString error = ioThread_->GetError();
-		if (error.empty())
+		std::wstring error = ioThread_->GetError();
+		if (error.empty()) {
 			controlSocket_.LogMessage(MessageType::Error, _("Can't write data to file."));
-		else
+		}
+		else {
 			controlSocket_.LogMessage(MessageType::Error, _("Can't write data to file: %s"), error);
+		}
 		TransferEnd(TransferEndReason::transfer_failure_critical);
 	}
 }
@@ -723,7 +752,7 @@ bool CTransferSocket::InitTls(const CTlsSocket* pPrimaryTlsSocket)
 	// Disable Nagle algorithm during TlS handshake
 	m_pSocket->SetFlags(m_pSocket->GetFlags() | CSocket::flag_nodelay);
 
-	wxASSERT(!m_pBackend);
+	assert(!m_pBackend);
 	m_pTlsSocket = new CTlsSocket(this, *m_pSocket, &controlSocket_);
 
 	if (!m_pTlsSocket->Init()) {
@@ -735,8 +764,7 @@ bool CTransferSocket::InitTls(const CTlsSocket* pPrimaryTlsSocket)
 	bool try_resume = CServerCapabilities::GetCapability(*controlSocket_.m_pCurrentServer, tls_resume) != no;
 
 	int res = m_pTlsSocket->Handshake(pPrimaryTlsSocket, try_resume);
-	if (res && res != FZ_REPLY_WOULDBLOCK)
-	{
+	if (res && res != FZ_REPLY_WOULDBLOCK) {
 		delete m_pTlsSocket;
 		m_pTlsSocket = 0;
 		return false;
@@ -749,30 +777,34 @@ bool CTransferSocket::InitTls(const CTlsSocket* pPrimaryTlsSocket)
 
 void CTransferSocket::TriggerPostponedEvents()
 {
-	wxASSERT(m_bActive);
+	assert(m_bActive);
 
 	if (m_postponedReceive) {
 		controlSocket_.LogMessage(MessageType::Debug_Verbose, _T("Executing postponed receive"));
 		m_postponedReceive = false;
 		OnReceive();
-		if (m_transferEndReason != TransferEndReason::none)
+		if (m_transferEndReason != TransferEndReason::none) {
 			return;
+		}
 	}
 	if (m_postponedSend) {
 		controlSocket_.LogMessage(MessageType::Debug_Verbose, _T("Executing postponed send"));
 		m_postponedSend = false;
 		OnSend();
-		if (m_transferEndReason != TransferEndReason::none)
+		if (m_transferEndReason != TransferEndReason::none) {
 			return;
+		}
 	}
-	if (m_onCloseCalled)
+	if (m_onCloseCalled) {
 		OnClose(0);
+	}
 }
 
 bool CTransferSocket::InitBackend()
 {
-	if (m_pBackend)
+	if (m_pBackend) {
 		return true;
+	}
 
 #ifdef FZ_WINDOWS
 	// For send buffer tuning
@@ -780,18 +812,23 @@ bool CTransferSocket::InitBackend()
 #endif
 
 	if (controlSocket_.m_protectDataChannel) {
-		if (!InitTls(controlSocket_.m_pTlsSocket))
+		if (!InitTls(controlSocket_.m_pTlsSocket)) {
 			return false;
+		}
 	}
-	else
+	else {
 		m_pBackend = new CSocketBackend(this, *m_pSocket, engine_.GetRateLimiter());
+	}
 
 	return true;
 }
 
 void CTransferSocket::SetSocketBufferSizes(CSocket* pSocket)
 {
-	wxCHECK_RET(pSocket, _("SetSocketBufferSize called without socket"));
+	if (!pSocket) {
+		controlSocket_.LogMessage(MessageType::Debug_Warning, L"SetSocketBufferSize called without socket");
+		return;
+	}
 
 	const int size_read = engine_.GetOptions().GetOptionVal(OPTION_SOCKET_BUFFERSIZE_RECV);
 #if FZ_WINDOWS
