@@ -9,7 +9,7 @@
 #include "buildinfo.h"
 #include "cmdline.h"
 #include "welcome_dialog.h"
-#include <msgbox.h>
+#include "msgbox.h"
 
 #include <libfilezilla/local_filesys.hpp>
 
@@ -39,10 +39,14 @@ IMPLEMENT_APP(CFileZillaApp)
 IMPLEMENT_APP_NO_MAIN(CFileZillaApp)
 #endif //__WXGTK__
 
+#if !wxUSE_PRINTF_POS_PARAMS
+  #error Please build wxWidgets with support for positional arguments.
+#endif
+
 CFileZillaApp::CFileZillaApp()
 {
 	m_profile_start = fz::monotonic_clock::now();
-	AddStartupProfileRecord(_T("CFileZillaApp::CFileZillaApp()"));
+	AddStartupProfileRecord("CFileZillaApp::CFileZillaApp()");
 }
 
 CFileZillaApp::~CFileZillaApp()
@@ -154,11 +158,27 @@ void CFileZillaApp::InitLocale()
 	}
 }
 
+namespace {
+std::wstring translator(char const* const t)
+{
+	return wxGetTranslation(t).ToStdWstring();
+}
+
+std::wstring translator_pf(char const* const singular, char const* const plural, int64_t n)
+{
+	// wxGetTranslation does not support 64bit ints on 32bit systems.
+	if (n < 0) {
+		n = -n;
+	}
+	return wxGetTranslation(singular, plural, (sizeof(unsigned int) < 8 && n > 1000000000) ? (1000000000 + n % 1000000000) : n).ToStdWstring();
+}
+}
+
 bool CFileZillaApp::OnInit()
 {
-	AddStartupProfileRecord(_T("CFileZillaApp::OnInit()"));
+	AddStartupProfileRecord("CFileZillaApp::OnInit()");
 
-	srand( (unsigned)time( NULL ) );
+	fz::set_translators(translator, translator_pf);
 
 #ifdef __WXMSW__
 	if (!InitWinsock()) {
@@ -271,34 +291,12 @@ int CFileZillaApp::OnExit()
 	return wxApp::OnExit();
 }
 
-bool CFileZillaApp::FileExists(const wxString& file) const
+bool CFileZillaApp::FileExists(std::wstring const& file) const
 {
-	int pos = file.Find('*');
-	if (pos < 0)
-		return wxFileExists(file);
-
-	wxASSERT(pos > 0);
-	wxASSERT(file[pos - 1] == '/');
-	wxASSERT(file.size() > static_cast<size_t>(pos + 1) && file[pos + 1] == '/');
-
-	wxLogNull nullLog;
-	wxDir dir(file.Left(pos));
-	if (!dir.IsOpened())
-		return false;
-
-	wxString subDir;
-	bool found = dir.GetFirst(&subDir, _T(""), wxDIR_DIRS);
-	while (found) {
-		if (FileExists(file.Left(pos) + subDir + file.Mid(pos + 1)))
-			return true;
-
-		found = dir.GetNext(&subDir);
-	}
-
-	return false;
+	return fz::local_filesys::get_file_type(fz::to_native(file), true) == fz::local_filesys::file;
 }
 
-CLocalPath CFileZillaApp::GetDataDir(wxString fileToFind) const
+CLocalPath CFileZillaApp::GetDataDir(std::wstring fileToFind) const
 {
 	/*
 	 * Finding the resources in all cases is a difficult task,
@@ -312,9 +310,10 @@ CLocalPath CFileZillaApp::GetDataDir(wxString fileToFind) const
 	 */
 
 #ifdef __WXMAC__
-	CLocalPath path(wxStandardPaths::Get().GetDataDir());
-	if (FileExists(path.GetPath() + fileToFind))
+	CLocalPath path(wxStandardPaths::Get().GetDataDir().ToStdWstring());
+	if (FileExists(path.GetPath() + fileToFind)) {
 		return path;
+	}
 
 	return CLocalPath();
 #else
@@ -366,27 +365,33 @@ CLocalPath CFileZillaApp::GetDataDir(wxString fileToFind) const
 	// For each path, check for the resources
 	wxPathList::const_iterator node;
 	for (node = pathList.begin(); node != pathList.end(); ++node) {
-		wxString cur = CLocalPath(*node).GetPath();
-		if (FileExists(cur + fileToFind))
+		auto const cur = CLocalPath(node->ToStdWstring()).GetPath();
+		if (FileExists(cur + fileToFind)) {
 			return CLocalPath(cur);
-		if (FileExists(cur + _T("share/filezilla/") + fileToFind))
+		}
+		if (FileExists(cur + _T("share/filezilla/") + fileToFind)) {
 			return CLocalPath(cur + _T("/share/filezilla"));
-		if (FileExists(cur + _T("filezilla/") + fileToFind))
+		}
+		if (FileExists(cur + _T("filezilla/") + fileToFind)) {
 			return CLocalPath(cur + _T("filezilla"));
+		}
 	}
 
 	for (node = pathList.begin(); node != pathList.end(); ++node) {
-		wxString cur = CLocalPath(*node).GetPath();
-		if (FileExists(cur + _T("../") + fileToFind))
+		auto const cur = CLocalPath(node->ToStdWstring()).GetPath();
+		if (FileExists(cur + _T("../") + fileToFind)) {
 			return CLocalPath(cur + _T("/.."));
-		if (FileExists(cur + _T("../share/filezilla/") + fileToFind))
+		}
+		if (FileExists(cur + _T("../share/filezilla/") + fileToFind)) {
 			return CLocalPath(cur + _T("../share/filezilla"));
+		}
 	}
 
 	for (node = pathList.begin(); node != pathList.end(); ++node) {
-		wxString cur = CLocalPath(*node).GetPath();
-		if (FileExists(cur + _T("../../") + fileToFind))
+		auto const cur = CLocalPath(node->ToStdWstring()).GetPath();
+		if (FileExists(cur + _T("../../") + fileToFind)) {
 			return CLocalPath(cur + _T("../.."));
+		}
 	}
 
 	return CLocalPath();
@@ -395,8 +400,8 @@ CLocalPath CFileZillaApp::GetDataDir(wxString fileToFind) const
 
 bool CFileZillaApp::LoadResourceFiles()
 {
-	AddStartupProfileRecord(_T("CFileZillaApp::LoadResourceFiles"));
-	m_resourceDir = GetDataDir(_T("resources/theme.xml"));
+	AddStartupProfileRecord("CFileZillaApp::LoadResourceFiles");
+	m_resourceDir = GetDataDir(_T("resources/defaultfilters.xml"));
 
 	wxImage::AddHandler(new wxPNGHandler());
 
@@ -416,23 +421,16 @@ bool CFileZillaApp::LoadResourceFiles()
 
 	InitHandlers(*pResource);
 
-	if (fz::local_filesys::get_file_type(fz::to_native(m_resourceDir.GetPath() + _T("xrc/resources.xrc"))) == fz::local_filesys::file) {
-		wxFileName fn(m_resourceDir.GetPath() + _T("xrc/resources.xrc"));
-		pResource->LoadFile(fn);
-	}
-	else {
-		fz::local_filesys fs;
-		wxString dir = m_resourceDir.GetPath() + _T("xrc/");
-		bool found = fs.begin_find_files(fz::to_native(dir), false);
-		while (found) {
-			fz::native_string name;
-			found = fs.get_next_file(name);
-			if (name.size() <= 4 || name.substr(name.size() - 4) != fzT(".xrc")) {
-				continue;
-			}
-			pResource->LoadFile(dir + name);
-		}
-	}
+	fz::local_filesys fs;
+	std::wstring dir = m_resourceDir.GetPath() + L"xrc/";
+	pResource->LoadFile(wxString(dir + L"certificate.xrc"));
+	pResource->LoadFile(wxString(dir + L"dialogs.xrc"));
+	pResource->LoadFile(wxString(dir + L"inputdialog.xrc"));
+	pResource->LoadFile(wxString(dir + L"menus.xrc"));
+	pResource->LoadFile(wxString(dir + L"netconfwizard.xrc"));
+	pResource->LoadFile(wxString(dir + L"settings.xrc"));
+	pResource->LoadFile(wxString(dir + L"sitemanager.xrc"));
+	pResource->LoadFile(wxString(dir + L"update.xrc"));
 
 	// Useful for XRC files with embedded image data.
 	wxFileSystem::AddHandler(new wxFileSystemBlobHandler);
@@ -442,7 +440,7 @@ bool CFileZillaApp::LoadResourceFiles()
 
 bool CFileZillaApp::InitDefaultsDir()
 {
-	AddStartupProfileRecord(_T("InitDefaultsDir"));
+	AddStartupProfileRecord("InitDefaultsDir");
 #ifdef __WXGTK__
 	m_defaultsDir = COptions::GetUnadjustedSettingsDir();
 	if( m_defaultsDir.empty() || !wxFileName::FileExists(m_defaultsDir.GetPath() + _T("fzdefaults.xml"))) {
@@ -464,16 +462,16 @@ bool CFileZillaApp::InitDefaultsDir()
 
 bool CFileZillaApp::LoadLocales()
 {
-	AddStartupProfileRecord(_T("CFileZillaApp::LoadLocales"));
+	AddStartupProfileRecord("CFileZillaApp::LoadLocales");
 #ifndef __WXMAC__
-	m_localesDir = GetDataDir(_T("../locale/*/filezilla.mo"));
+	m_localesDir = GetDataDir(_T("../locale/de/filezilla.mo"));
 	if (m_localesDir.empty())
-		m_localesDir = GetDataDir(_T("../locale/*/LC_MESSAGES/filezilla.mo"));
+		m_localesDir = GetDataDir(_T("../locale/de/LC_MESSAGES/filezilla.mo"));
 	if (!m_localesDir.empty()) {
 		m_localesDir.ChangePath( _T("../locale") );
 	}
 	else {
-		m_localesDir = GetDataDir(_T("locales/*/filezilla.mo"));
+		m_localesDir = GetDataDir(_T("locales/de/filezilla.mo"));
 		if (!m_localesDir.empty()) {
 			m_localesDir.AddSegment(_T("locales"));
 		}
@@ -567,7 +565,7 @@ CWrapEngine* CFileZillaApp::GetWrapEngine()
 
 void CFileZillaApp::CheckExistsFzsftp()
 {
-	AddStartupProfileRecord(_T("CFileZillaApp::CheckExistsFzstp"));
+	AddStartupProfileRecord("FileZillaApp::CheckExistsFzstp");
 	// Get the correct path to the fzsftp executable
 
 #ifdef __WXMAC__
@@ -576,7 +574,7 @@ void CFileZillaApp::CheckExistsFzsftp()
 	if (pos != -1)
 		executable = executable.Left(pos);
 	executable += _T("/fzsftp");
-	if (!wxFileName::FileExists(executable))
+	if (!wxFileName::FileExists(executable.ToStdWstring()))
 	{
 		wxMessageBoxEx(wxString::Format(_("%s could not be found. Without this component of FileZilla, SFTP will not work.\n\nPlease download FileZilla again. If this problem persists, please submit a bug report."), executable),
 			_("File not found"), wxICON_ERROR);
@@ -595,7 +593,7 @@ void CFileZillaApp::CheckExistsFzsftp()
 	// First check the FZ_FZSFTP environment variable
 	wxString executable;
 	if (wxGetEnv(_T("FZ_FZSFTP"), &executable)) {
-		if (wxFileName::FileExists(executable))
+		if (wxFileName::FileExists(executable.ToStdWstring()))
 			found = true;
 	}
 
@@ -686,7 +684,7 @@ extern "C" BOOL CALLBACK EnumWindowCallback(HWND hwnd, LPARAM)
 
 int CFileZillaApp::ProcessCommandLine()
 {
-	AddStartupProfileRecord(_T("CFileZillaApp::ProcessCommandLine"));
+	AddStartupProfileRecord("CFileZillaApp::ProcessCommandLine");
 	m_pCommandLine = std::make_unique<CCommandLine>(argc, argv);
 	int res = m_pCommandLine->Parse() ? 1 : -1;
 
@@ -712,7 +710,7 @@ int CFileZillaApp::ProcessCommandLine()
 	return res;
 }
 
-void CFileZillaApp::AddStartupProfileRecord(const wxString& msg)
+void CFileZillaApp::AddStartupProfileRecord(std::string const& msg)
 {
 	if (!m_profile_start) {
 		return;
@@ -730,7 +728,7 @@ void CFileZillaApp::ShowStartupProfile()
 
 			msg += std::to_wstring(diff.get_milliseconds());
 			msg += _T(" ");
-			msg += p.second;
+			msg += fz::to_wstring(p.second);
 			msg += _T("\n");
 		}
 		wxMessageBoxEx(msg);
@@ -740,7 +738,7 @@ void CFileZillaApp::ShowStartupProfile()
 	m_startupProfile.clear();
 }
 
-wxString CFileZillaApp::GetSettingsFile(wxString const& name) const
+std::wstring CFileZillaApp::GetSettingsFile(std::wstring const& name) const
 {
 	return COptions::Get()->GetOption(OPTION_DEFAULT_SETTINGSDIR) + name + _T(".xml");
 }
