@@ -11,6 +11,7 @@
 #include "LocalTreeView.h"
 #include "Options.h"
 #include "queue.h"
+#include "themeprovider.h"
 
 #include <libfilezilla/local_filesys.hpp>
 
@@ -59,8 +60,9 @@ public:
 		const wxString& dir = m_pLocalTreeView->GetDirFromItem(item);
 
 #ifdef __WXMSW__
-		if (dir == _T("/"))
+		if (dir == _T("/")) {
 			return wxString();
+		}
 #endif
 
 		return dir;
@@ -71,8 +73,9 @@ public:
 		int flags = 0;
 		wxTreeItemId hit = m_pLocalTreeView->HitTest(point, flags);
 
-		if (flags & (wxTREE_HITTEST_ABOVE | wxTREE_HITTEST_BELOW | wxTREE_HITTEST_NOWHERE | wxTREE_HITTEST_TOLEFT | wxTREE_HITTEST_TORIGHT))
+		if (flags & (wxTREE_HITTEST_ABOVE | wxTREE_HITTEST_BELOW | wxTREE_HITTEST_NOWHERE | wxTREE_HITTEST_TOLEFT | wxTREE_HITTEST_TORIGHT)) {
 			return wxTreeItemId();
+		}
 
 		return hit;
 	}
@@ -91,7 +94,7 @@ public:
 		if (!hit)
 			return wxDragNone;
 
-		const CLocalPath path(GetDirFromItem(hit));
+		const CLocalPath path(GetDirFromItem(hit).ToStdWstring());
 		if (path.empty() || !path.IsWriteable())
 			return wxDragNone;
 
@@ -134,12 +137,14 @@ public:
 		ClearDropHighlight();
 
 		wxTreeItemId hit = GetHit(wxPoint(x, y));
-		if (!hit)
+		if (!hit) {
 			return false;
+		}
 
-		const wxString dir = GetDirFromItem(hit);
-		if (dir.empty() || !CLocalPath(dir).IsWriteable())
+		std::wstring const dir = GetDirFromItem(hit).ToStdWstring();
+		if (dir.empty() || !CLocalPath(dir).IsWriteable()) {
 			return false;
+		}
 
 		return true;
 	}
@@ -239,11 +244,11 @@ END_EVENT_TABLE()
 
 CLocalTreeView::CLocalTreeView(wxWindow* parent, wxWindowID id, CState& state, CQueueView *pQueueView)
 	: wxTreeCtrlEx(parent, id, wxDefaultPosition, wxDefaultSize, wxTAB_TRAVERSAL | wxTR_EDIT_LABELS | wxTR_LINES_AT_ROOT | wxTR_HAS_BUTTONS | wxNO_BORDER),
-	CSystemImageList(16),
+	CSystemImageList(CThemeProvider::GetIconSize(iconSizeSmall).x),
 	CStateEventHandler(state),
 	m_pQueueView(pQueueView)
 {
-	wxGetApp().AddStartupProfileRecord(_T("CLocalTreeView::CLocalTreeView"));
+	wxGetApp().AddStartupProfileRecord("CLocalTreeView::CLocalTreeView");
 #ifdef __WXMAC__
 	SetFont(wxSystemSettings::GetFont(wxSYS_DEFAULT_GUI_FONT));
 #endif
@@ -402,7 +407,7 @@ wxTreeItemId CLocalTreeView::GetSubdir(wxTreeItemId parent, const wxString& subD
 
 bool CLocalTreeView::DisplayDrives(wxTreeItemId parent)
 {
-	wxGetApp().AddStartupProfileRecord(_T("CLocalTreeView::DisplayDrives"));
+	wxGetApp().AddStartupProfileRecord("CLocalTreeView::DisplayDrives");
 
 	std::list<std::wstring> drives = CVolumeDescriptionEnumeratorThread::GetDrives();
 
@@ -422,7 +427,7 @@ bool CLocalTreeView::DisplayDrives(wxTreeItemId parent)
 	}
 	SortChildren(parent);
 
-	wxGetApp().AddStartupProfileRecord(_T("CLocalTreeView::DisplayDrives adding drives done"));
+	wxGetApp().AddStartupProfileRecord("CLocalTreeView::DisplayDrives adding drives done");
 
 	return true;
 }
@@ -733,7 +738,7 @@ void CLocalTreeView::RefreshListing()
 		}
 
 		// Step 2: Enumerate subdirectories on disk and sort them
-		std::vector<wxString> dirs;
+		std::vector<std::wstring> dirs;
 
 		fz::native_string file;
 		static int64_t const size(-1);
@@ -752,7 +757,7 @@ void CLocalTreeView::RefreshListing()
 				continue;
 			}
 
-			dirs.push_back(wfile);
+			dirs.emplace_back(std::move(wfile));
 		}
 		auto const& sortFunc = CFileListCtrlSortBase::GetCmpFunction(m_nameSortMode);
 		std::sort(dirs.begin(), dirs.end(), [&](auto const& lhs, auto const& rhs) { return sortFunc(lhs, rhs) < 0; });
@@ -840,17 +845,20 @@ void CLocalTreeView::OnSelectionChanged(wxTreeEvent& event)
 	}
 
 	wxTreeItemId item = event.GetItem();
-	if (!item)
+	if (!item) {
 		return;
+	}
 
-	wxString dir = GetDirFromItem(item);
+	std::wstring dir = GetDirFromItem(item).ToStdWstring();
 
-	wxString error;
+	std::wstring error;
 	if (!m_state.SetLocalDir(dir, &error)) {
-		if (!error.empty())
+		if (!error.empty()) {
 			wxMessageBoxEx(error, _("Failed to change directory"), wxICON_INFORMATION);
-		else
+		}
+		else {
 			wxBell();
+		}
 		m_setSelection = true;
 		SelectItem(event.GetOldItem());
 		m_setSelection = false;
@@ -1030,14 +1038,16 @@ void CLocalTreeView::OnVolumesEnumerated(wxCommandEvent& event)
 void CLocalTreeView::OnContextMenu(wxTreeEvent& event)
 {
 	m_contextMenuItem = event.GetItem();
-	if (!m_contextMenuItem.IsOk())
+	if (!m_contextMenuItem.IsOk()) {
 		return;
+	}
 
 	wxMenu* pMenu = wxXmlResource::Get()->LoadMenu(_T("ID_MENU_LOCALTREE"));
-	if (!pMenu)
+	if (!pMenu) {
 		return;
+	}
 
-	const CLocalPath path(GetDirFromItem(m_contextMenuItem));
+	CLocalPath const path(GetDirFromItem(m_contextMenuItem).ToStdWstring());
 
 	const bool hasParent = path.HasParent();
 	const bool writeable = path.IsWriteable();
@@ -1063,29 +1073,34 @@ void CLocalTreeView::OnMenuUpload(wxCommandEvent& event)
 		return;
 	}
 
-	if (!m_contextMenuItem.IsOk())
+	if (!m_contextMenuItem.IsOk()) {
 		return;
+	}
 
-	CLocalPath path(GetDirFromItem(m_contextMenuItem));
+	CLocalPath path(GetDirFromItem(m_contextMenuItem).ToStdWstring());
 
-	if (!path.HasParent())
+	if (!path.HasParent()) {
 		return;
+	}
 
-	if (!m_state.IsRemoteConnected())
+	if (!m_state.IsRemoteConnected()) {
 		return;
+	}
 
 	const CServer server = *m_state.GetServer();
 	CServerPath remotePath = m_state.GetRemotePath();
-	if (remotePath.empty())
+	if (remotePath.empty()) {
 		return;
+	}
 
-	if (!remotePath.ChangePath(GetItemText(m_contextMenuItem)))
+	if (!remotePath.ChangePath(GetItemText(m_contextMenuItem).ToStdWstring())) {
 		return;
+	}
 
 	local_recursion_root root;
 	root.add_dir_to_visit(path, remotePath);
 	recursiveOperation->AddRecursionRoot(std::move(root));
-	
+
 	bool const queue_only = event.GetId() == XRCID("ID_ADDTOQUEUE");
 
 	CFilterManager filter;
@@ -1105,19 +1120,20 @@ void CLocalTreeView::OnMenuMkdir(wxCommandEvent&)
 // Create a new Directory and enter the new Directory
 void CLocalTreeView::OnMenuMkdirChgDir(wxCommandEvent&)
 {
-	wxString newdir = MenuMkdir();
+	std::wstring newdir = MenuMkdir().ToStdWstring();
 	if (newdir.empty()) {
 		return;
 	}
 
 	// OnMenuEnter
-	wxString error;
-	if (!m_state.SetLocalDir(newdir, &error))
-	{
-		if (!error.empty())
+	std::wstring error;
+	if (!m_state.SetLocalDir(newdir, &error)) {
+		if (!error.empty()) {
 			wxMessageBoxEx(error, _("Failed to change directory"), wxICON_INFORMATION);
-		else
+		}
+		else {
 			wxBell();
+		}
 	}
 }
 
@@ -1125,33 +1141,35 @@ void CLocalTreeView::OnMenuMkdirChgDir(wxCommandEvent&)
 // Returns the name of the new directory
 wxString CLocalTreeView::MenuMkdir()
 {
-	if (!m_contextMenuItem.IsOk())
+	if (!m_contextMenuItem.IsOk()) {
 		return wxString();
+	}
 
-	wxString path = GetDirFromItem(m_contextMenuItem);
-	if (path.Last() != wxFileName::GetPathSeparator())
+	std::wstring path = GetDirFromItem(m_contextMenuItem).ToStdWstring();
+	if (!path.empty() && path.back() != wxFileName::GetPathSeparator()) {
 		path += wxFileName::GetPathSeparator();
+	}
 
-	if (!CLocalPath(path).IsWriteable())
-	{
+	if (!CLocalPath(path).IsWriteable()) {
 		wxBell();
 		return wxString();
 	}
 
 	CInputDialog dlg;
-	if (!dlg.Create(this, _("Create directory"), _("Please enter the name of the directory which should be created:")))
+	if (!dlg.Create(this, _("Create directory"), _("Please enter the name of the directory which should be created:"))) {
 		return wxString();
+	}
 
 	wxString newName = _("New directory");
 	dlg.SetValue(path + newName);
-	dlg.SelectText(path.Len(), path.Len() + newName.Len());
+	dlg.SelectText(path.size(), path.size() + newName.size());
 
-	if (dlg.ShowModal() != wxID_OK)
+	if (dlg.ShowModal() != wxID_OK) {
 		return wxString();
+	}
 
 	wxFileName fn(dlg.GetValue(), _T(""));
-	if (!fn.Normalize(wxPATH_NORM_ALL, path))
-	{
+	if (!fn.Normalize(wxPATH_NORM_ALL, path)) {
 		wxBell();
 		return wxString();
 	}
@@ -1182,7 +1200,7 @@ void CLocalTreeView::OnMenuRename(wxCommandEvent&)
 	}
 #endif
 
-	CLocalPath path(GetDirFromItem(m_contextMenuItem));
+	CLocalPath path(GetDirFromItem(m_contextMenuItem).ToStdWstring());
 	if (!path.HasParent() || !path.IsWriteable()) {
 		wxBell();
 		return;
@@ -1193,37 +1211,45 @@ void CLocalTreeView::OnMenuRename(wxCommandEvent&)
 
 void CLocalTreeView::OnMenuDelete(wxCommandEvent&)
 {
-	if (!m_contextMenuItem.IsOk())
+	if (!m_contextMenuItem.IsOk()) {
 		return;
+	}
 
-	wxString path = GetDirFromItem(m_contextMenuItem);
+	std::wstring path = GetDirFromItem(m_contextMenuItem).ToStdWstring();
 
 	CLocalPath local_path(path);
-	if (!local_path.HasParent() || !local_path.IsWriteable())
+	if (!local_path.HasParent() || !local_path.IsWriteable()) {
 		return;
+	}
 
 	gui_recursive_remove rmd(this);
 	rmd.remove(fz::to_native(path));
 
 	wxTreeItemId item = GetSelection();
-	while (item && item != m_contextMenuItem)
+	while (item && item != m_contextMenuItem) {
 		item = GetItemParent(item);
+	}
 
 	if (!item) {
-		if (GetItemParent(m_contextMenuItem) == GetSelection())
+		if (GetItemParent(m_contextMenuItem) == GetSelection()) {
 			m_state.RefreshLocal();
-		else
+		}
+		else {
 			RefreshListing();
+		}
 		return;
 	}
 
-	if (!path.empty() && path.Last() == wxFileName::GetPathSeparator())
-		path.RemoveLast();
-	int pos = path.Find(wxFileName::GetPathSeparator(), true);
-	if (pos < 1)
+	if (!path.empty() && path.back() == wxFileName::GetPathSeparator()) {
+		path.pop_back();
+	}
+	size_t pos = path.rfind(wxFileName::GetPathSeparator());
+	if (pos == std::wstring::npos || !pos) {
 		path = _T("/");
-	else
-		path = path.Left(pos);
+	}
+	else {
+		path = path.substr(0, pos);
+	}
 
 	m_state.SetLocalDir(path);
 	RefreshListing();
@@ -1242,7 +1268,7 @@ void CLocalTreeView::OnBeginLabelEdit(wxTreeEvent& event)
 	}
 #endif
 
-	CLocalPath path(GetDirFromItem(item));
+	CLocalPath path(GetDirFromItem(item).ToStdWstring());
 
 	if (!path.HasParent() || !path.IsWriteable())
 	{
@@ -1269,7 +1295,7 @@ void CLocalTreeView::OnEndLabelEdit(wxTreeEvent& event)
 	}
 #endif
 
-	wxString path = GetDirFromItem(item);
+	std::wstring path = GetDirFromItem(item).ToStdWstring();
 
 	CLocalPath local_path(path);
 	if (!local_path.HasParent() || !local_path.IsWriteable()) {
@@ -1278,16 +1304,17 @@ void CLocalTreeView::OnEndLabelEdit(wxTreeEvent& event)
 		return;
 	}
 
-	if (!path.empty() && path.Last() == wxFileName::GetPathSeparator())
-		path.RemoveLast();
+	if (!path.empty() && path.back() == wxFileName::GetPathSeparator()) {
+		path.pop_back();
+	}
 
-	int pos = path.Find(wxFileName::GetPathSeparator(), true);
-	wxASSERT(pos != -1);
+	size_t pos = path.rfind(wxFileName::GetPathSeparator());
+	wxASSERT(pos != std::wstring::npos);
 
-	wxString parent = path.Left(pos + 1);
+	std::wstring parent = path.substr(0, pos + 1);
 
 	const wxString& oldName = GetItemText(item);
-	const wxString& newName = event.GetLabel();
+	std::wstring const newName = event.GetLabel().ToStdWstring();
 	if (newName.empty()) {
 		wxBell();
 		event.Veto();
@@ -1296,8 +1323,9 @@ void CLocalTreeView::OnEndLabelEdit(wxTreeEvent& event)
 
 	wxASSERT(parent + oldName == path);
 
-	if (oldName == newName)
+	if (oldName == newName) {
 		return;
+	}
 
 	if (!RenameFile(this, parent, oldName, newName)) {
 		event.Veto();
@@ -1321,11 +1349,11 @@ void CLocalTreeView::OnEndLabelEdit(wxTreeEvent& event)
 		return;
 	}
 
-	wxString sub;
+	std::wstring sub;
 
 	wxTreeItemId tmp = currentSel;
 	while (tmp != GetRootItem() && tmp != item) {
-		sub = wxFileName::GetPathSeparator() + GetItemText(tmp) + sub;
+		sub = (wxFileName::GetPathSeparator() + GetItemText(tmp) + sub).ToStdWstring();
 		tmp = GetItemParent(tmp);
 	}
 
@@ -1376,8 +1404,9 @@ bool CLocalTreeView::CheckSubdirStatus(wxTreeItemId& item, const wxString& path)
 #endif
 
 	if (child) {
-		if (!GetItemText(child).empty())
+		if (!GetItemText(child).empty()) {
 			return false;
+		}
 
 		CTreeItemData* pData = (CTreeItemData*)GetItemData(child);
 		if (pData) {
@@ -1385,10 +1414,12 @@ bool CLocalTreeView::CheckSubdirStatus(wxTreeItemId& item, const wxString& path)
 			int attributes;
 			fz::local_filesys::type type;
 			fz::datetime date;
-			if (!path.empty() && path.Last() == fz::local_filesys::path_separator)
+			if (!path.empty() && path.Last() == fz::local_filesys::path_separator) {
 				type = fz::local_filesys::get_file_info(fz::to_native(path + pData->m_known_subdir), wasLink, 0, &date, &attributes);
-			else
+			}
+			else {
 				type = fz::local_filesys::get_file_info(fz::to_native(path + fz::local_filesys::path_separator + pData->m_known_subdir), wasLink, 0, &date, &attributes);
+			}
 			if (type == fz::local_filesys::dir) {
 				CFilterManager filter;
 				if (!filter.FilenameFiltered(pData->m_known_subdir, path, true, size, true, attributes, date))
