@@ -108,6 +108,7 @@ static const t_Option options[OPTIONS_NUM] =
 	{ "Size thousands separator", number, _T("1"), normal },
 	{ "Size decimal places", number, _T("1"), normal },
 	{ "TCP Keepalive Interval", number, _T("15"), normal },
+	{ "Cache TTL", number, _T("600"), normal },
 
 	// Interface settings
 	{ "Number of Transfers", number, _T("2"), normal },
@@ -115,7 +116,6 @@ static const t_Option options[OPTIONS_NUM] =
 	{ "Auto Ascii files", string, _T("am|asp|bat|c|cfm|cgi|conf|cpp|css|dhtml|diz|h|hpp|htm|html|in|inc|java|js|jsp|lua|m4|mak|md5|nfo|nsi|pas|patch|php|phtml|pl|po|py|qmail|sh|sha1|sha256|sha512|shtml|sql|svg|tcl|tpl|txt|vbs|xhtml|xml|xrc"), normal },
 	{ "Auto Ascii no extension", number, _T("1"), normal },
 	{ "Auto Ascii dotfiles", number, _T("1"), normal },
-	{ "Theme", string, _T("opencrystal/"), normal },
 	{ "Language Code", string, _T(""), normal },
 	{ "Last Server Path", string, _T(""), normal },
 	{ "Concurrent download limit", number, _T("0"), normal },
@@ -159,7 +159,8 @@ static const t_Option options[OPTIONS_NUM] =
 	{ "Comparison mode", number, _T("1"), normal },
 	{ "Comparison threshold", number, _T("1"), normal },
 	{ "Site Manager position", string, _T(""), normal },
-	{ "Theme icon size", string, _T(""), normal },
+	{ "Icon theme", string, _T("default"), normal },
+	{ "Icon scale", number, _T("125"), normal },
 	{ "Timestamp in message log", number, _T("0"), normal },
 	{ "Sitemanager last selected", string, _T(""), normal },
 	{ "Local filelist shown columns", string, _T(""), normal },
@@ -201,7 +202,8 @@ static const t_Option options[OPTIONS_NUM] =
 	// Default/internal options
 	{ "Config Location", string, _T(""), default_only },
 	{ "Kiosk mode", number, _T("0"), default_priority },
-	{ "Disable update check", number, _T("0"), default_only }
+	{ "Disable update check", number, _T("0"), default_only },
+	{ "Cache directory", string, _T(""), default_priority },
 };
 
 BEGIN_EVENT_TABLE(COptions, wxEvtHandler)
@@ -245,8 +247,9 @@ COptions::COptions()
 		delete m_pXmlFile;
 		m_pXmlFile = 0;
 	}
-	else
+	else {
 		CreateSettingsXmlElement();
+	}
 
 	LoadOptions(nameOptionMap);
 }
@@ -255,8 +258,9 @@ std::map<std::string, unsigned int> COptions::GetNameOptionMap() const
 {
 	std::map<std::string, unsigned int> ret;
 	for (unsigned int i = 0; i < OPTIONS_NUM; ++i) {
-		if (options[i].flags != internal)
+		if (options[i].flags != internal) {
 			ret.insert(std::make_pair(std::string(options[i].name), i));
+		}
 	}
 	return ret;
 }
@@ -339,8 +343,9 @@ void COptions::ContinueSetOption(unsigned int nID, T const& value)
 	if (options[nID].flags == normal || options[nID].flags == default_priority) {
 		SetXmlValue(nID, validated);
 
-		if (!m_save_timer.IsRunning())
+		if (!m_save_timer.IsRunning()) {
 			m_save_timer.Start(15000, true);
+		}
 	}
 
 	if (changedOptions_.none()) {
@@ -359,8 +364,9 @@ void COptions::NotifyChangedOptions()
 
 bool COptions::OptionFromFzDefaultsXml(unsigned int nID)
 {
-	if (nID >= OPTIONS_NUM)
+	if (nID >= OPTIONS_NUM) {
 		return false;
+	}
 
 	fz::scoped_lock l(m_sync_);
 	return m_optionsCache[nID].from_default;
@@ -368,8 +374,9 @@ bool COptions::OptionFromFzDefaultsXml(unsigned int nID)
 
 pugi::xml_node COptions::CreateSettingsXmlElement()
 {
-	if (!m_pXmlFile)
+	if (!m_pXmlFile) {
 		return pugi::xml_node();
+	}
 
 	auto element = m_pXmlFile->GetElement();
 	if (!element) {
@@ -413,10 +420,12 @@ void COptions::SetXmlValue(unsigned int nID, std::wstring const& value)
 		pugi::xml_node setting;
 		for (setting = settings.child("Setting"); setting; setting = setting.next_sibling("Setting")) {
 			const char *attribute = setting.attribute("name").value();
-			if (!attribute)
+			if (!attribute) {
 				continue;
-			if (!strcmp(attribute, options[nID].name))
+			}
+			if (!strcmp(attribute, options[nID].name)) {
 				break;
+			}
 		}
 		if (!setting) {
 			setting = settings.append_child("Setting");
@@ -524,6 +533,14 @@ int COptions::Validate(unsigned int nID, int value)
 			value = 9999;
 		}
 		break;
+	case OPTION_CACHE_TTL:
+		if (value < 30) {
+			value = 30;
+		}
+		else if (value > 60*60*24) {
+			value = 60 * 60 * 24;
+		}
+		break;
 	}
 	return value;
 }
@@ -693,14 +710,16 @@ void COptions::LoadOptions(std::map<std::string, unsigned int> const& nameOption
 void COptions::LoadOptionFromElement(pugi::xml_node option, std::map<std::string, unsigned int> const& nameOptionMap, bool allowDefault)
 {
 	const char* name = option.attribute("name").value();
-	if (!name)
+	if (!name) {
 		return;
+	}
 
 	auto const iter = nameOptionMap.find(name);
 	if (iter != nameOptionMap.end()) {
-		if (!allowDefault && options[iter->second].flags == default_only)
+		if (!allowDefault && options[iter->second].flags == default_only) {
 			return;
-		std::wstring value = GetTextElement(option).ToStdWstring();
+		}
+		std::wstring value = GetTextElement(option);
 		if (options[iter->second].flags == default_priority) {
 			if (allowDefault) {
 				fz::scoped_lock l(m_sync_);
@@ -708,8 +727,9 @@ void COptions::LoadOptionFromElement(pugi::xml_node option, std::map<std::string
 			}
 			else {
 				fz::scoped_lock l(m_sync_);
-				if (m_optionsCache[iter->second].from_default)
+				if (m_optionsCache[iter->second].from_default) {
 					return;
+				}
 			}
 		}
 
@@ -730,20 +750,23 @@ void COptions::LoadOptionFromElement(pugi::xml_node option, std::map<std::string
 void COptions::LoadGlobalDefaultOptions(std::map<std::string, unsigned int> const& nameOptionMap)
 {
 	CLocalPath const defaultsDir = wxGetApp().GetDefaultsDir();
-	if (defaultsDir.empty())
+	if (defaultsDir.empty()) {
 		return;
-
+	}
 	CXmlFile file(defaultsDir.GetPath() + _T("fzdefaults.xml"));
-	if (!file.Load())
+	if (!file.Load()) {
 		return;
+	}
 
 	auto element = file.GetElement();
-	if (!element)
+	if (!element) {
 		return;
+	}
 
 	element = element.child("Settings");
-	if (!element)
+	if (!element) {
 		return;
+	}
 
 	for (auto setting = element.child("Setting"); setting; setting = setting.next_sibling("Setting")) {
 		LoadOptionFromElement(setting, nameOptionMap, true);
@@ -757,11 +780,13 @@ void COptions::OnTimer(wxTimerEvent&)
 
 void COptions::Save()
 {
-	if (GetOptionVal(OPTION_DEFAULT_KIOSKMODE) == 2)
+	if (GetOptionVal(OPTION_DEFAULT_KIOSKMODE) == 2) {
 		return;
+	}
 
-	if (!m_pXmlFile)
+	if (!m_pXmlFile) {
 		return;
+	}
 
 	CInterProcessMutex mutex(MUTEX_OPTIONS);
 	m_pXmlFile->Save(true);
@@ -769,8 +794,9 @@ void COptions::Save()
 
 void COptions::SaveIfNeeded()
 {
-	if (!m_save_timer.IsRunning())
+	if (!m_save_timer.IsRunning()) {
 		return;
+	}
 
 	m_save_timer.Stop();
 	Save();
@@ -781,14 +807,14 @@ namespace {
 std::wstring TryDirectory(wxString path, wxString const& suffix, bool check_exists)
 {
 	if (!path.empty() && path[0] == '/') {
-		if( path[path.size() - 1] != '/' ) {
+		if (path[path.size() - 1] != '/') {
 			path += '/';
 		}
 
 		path += suffix;
 
-		if( check_exists ) {
-			if( !wxFileName::DirExists(path) ) {
+		if (check_exists) {
+			if (!wxFileName::DirExists(path)) {
 				path.clear();
 			}
 		}
@@ -802,7 +828,7 @@ std::wstring TryDirectory(wxString path, wxString const& suffix, bool check_exis
 wxString GetEnv(wxString const& env)
 {
 	wxString ret;
-	if( !wxGetEnv(env, &ret) ) {
+	if (!wxGetEnv(env, &ret)) {
 		ret.clear();
 	}
 	return ret;
@@ -824,11 +850,12 @@ CLocalPath COptions::GetUnadjustedSettingsDir()
 		}
 		ret = tmp;
 	}
-	else {
+
+	if (ret.empty()) {
 		// Fall back to directory where the executable is
 		DWORD c = GetModuleFileName(0, buffer, MAX_PATH * 2);
 		if (c && c < MAX_PATH * 2) {
-			wxString tmp;
+			std::wstring tmp;
 			ret.SetPath(buffer, &tmp);
 		}
 	}
@@ -854,6 +881,33 @@ CLocalPath COptions::GetUnadjustedSettingsDir()
 	return ret;
 }
 
+CLocalPath COptions::GetCacheDirectory()
+{
+	CLocalPath dir(GetOption(OPTION_DEFAULT_CACHE_DIR));
+	if (!dir.empty()) {
+		return dir;
+	}
+
+#ifdef FZ_WINDOWS
+	wchar_t buffer[MAX_PATH * 2 + 1];
+
+	if (SUCCEEDED(SHGetFolderPath(0, CSIDL_LOCAL_APPDATA, 0, SHGFP_TYPE_CURRENT, buffer))) {
+		dir.SetPath(buffer);
+		if (!dir.empty()) {
+			dir.AddSegment(L"FileZilla");
+		}
+	}
+#else
+	std::wstring cfg = TryDirectory(GetEnv(_T("XDG_CACHE_HOME")), _T("filezilla/"), false);
+	if (cfg.empty()) {
+		cfg = TryDirectory(wxGetHomeDir(), _T(".cache/filezilla/"), false);
+	}
+	dir.SetPath(cfg);
+#endif
+
+	return dir;
+}
+
 CLocalPath COptions::InitSettingsDir()
 {
 	CLocalPath p;
