@@ -14,10 +14,13 @@
 #include "remote_recursive_operation.h"
 #include "themeprovider.h"
 
+#include "uri.h"
+
 #include <wx/clipbrd.h>
 
 #include <algorithm>
 
+
 class CItemData : public wxTreeItemData
 {
 public:
@@ -239,7 +242,7 @@ END_EVENT_TABLE()
 
 CRemoteTreeView::CRemoteTreeView(wxWindow* parent, wxWindowID id, CState& state, CQueueView* pQueue)
 	: wxTreeCtrlEx(parent, id, wxDefaultPosition, wxDefaultSize, wxTAB_TRAVERSAL | wxTR_EDIT_LABELS | wxTR_LINES_AT_ROOT | wxTR_HAS_BUTTONS | wxNO_BORDER | wxTR_HIDE_ROOT),
-	CSystemImageList(16),
+	CSystemImageList(CThemeProvider::GetIconSize(iconSizeSmall).x),
 	CStateEventHandler(state)
 {
 #ifdef __WXMAC__
@@ -439,41 +442,14 @@ wxTreeItemId CRemoteTreeView::MakeParent(CServerPath path, bool select)
 	return parent;
 }
 
-wxBitmap CRemoteTreeView::CreateIcon(int index, const wxString& overlay /*=_T("")*/)
+wxBitmap CRemoteTreeView::CreateIcon(int index, const wxString& overlay)
 {
-	// Create memory DC
-	wxSize s = CThemeProvider::GetIconSize(iconSizeSmall);
-#ifdef __WXMSW__
-	wxBitmap bmp(s.x, s.y, 32);
-#else
-	wxBitmap bmp(s.x, s.y, 24);
-#endif
-	wxMemoryDC dc;
-	dc.SelectObject(bmp);
-
-	// Make sure the background is set correctly
-	dc.SetBrush(wxBrush(GetBackgroundColour()));
-	dc.SetPen(wxPen(GetBackgroundColour()));
-	dc.DrawRectangle(0, 0, s.x, s.y);
-
-	// Draw item from system image list
-	GetSystemImageList()->Draw(index, dc, 0, 0, wxIMAGELIST_DRAW_TRANSPARENT);
-
-	// Load overlay
+	wxBitmap bmp = GetSystemImageList()->GetBitmap(index);
 	if (!overlay.empty()) {
-		wxImage unknownIcon = wxArtProvider::GetBitmap(overlay, wxART_OTHER, CThemeProvider::GetIconSize(iconSizeSmall)).ConvertToImage();
-
-		// Convert mask into alpha channel
-		if (unknownIcon.IsOk() && !unknownIcon.HasAlpha()) {
-			wxASSERT(unknownIcon.HasMask());
-			unknownIcon.InitAlpha();
-		}
-
-		// Draw overlay
-		dc.DrawBitmap(unknownIcon, 0, 0, true);
+		wxBitmap unknown = CThemeProvider::Get()->CreateBitmap(overlay, wxART_OTHER, bmp.GetScaledSize());
+		Overlay(bmp, unknown);
 	}
 
-	dc.SelectObject(wxNullBitmap);
 	return bmp;
 }
 
@@ -1388,8 +1364,9 @@ void CRemoteTreeView::ApplyFilters(bool resort)
 
 void CRemoteTreeView::OnMenuGeturl(wxCommandEvent& event)
 {
-	if (!m_contextMenuItem)
+	if (!m_contextMenuItem) {
 		return;
+	}
 
 	const CServerPath& path = GetPathFromItem(m_contextMenuItem);
 	if (path.empty()) {
@@ -1410,7 +1387,7 @@ void CRemoteTreeView::OnMenuGeturl(wxCommandEvent& event)
 
 	wxString url = pServer->Format((event.GetId() == XRCID("ID_GETURL_PASSWORD")) ? ServerFormat::url_with_password : ServerFormat::url);
 
-	std::wstring const pathPart = url_encode(path.GetPath(), true);
+	std::wstring const pathPart = fz::percent_encode_w(path.GetPath(), true);
 	if (!pathPart.empty() && pathPart[0] != '/') {
 		url += '/';
 	}
