@@ -17,21 +17,24 @@ public:
 		confirmexit,
 		sitemanager_confirmdelete,
 		confirmexit_edit, // Confirm closing FileZilla while files are still being edited
-		confirm_preserve_timestamps
+		confirm_preserve_timestamps,
+		compare_treeviewmismatch,
+		compare_changesorting,
+		many_selected_for_edit
 	};
 
-	CConditionalDialog(wxWindow* parent, enum DialogType type, enum Modes mode, bool checked = false);
+	CConditionalDialog(wxWindow* parent, DialogType type, Modes mode, bool checked = false);
 
 	void AddText(const wxString &text);
 
 	bool Run();
 
 protected:
-	enum DialogType m_type;
+	DialogType m_type;
 
 	wxSizer* m_pTextSizer;
 
-	DECLARE_EVENT_TABLE();
+	DECLARE_EVENT_TABLE()
 	void OnButton(wxCommandEvent& event);
 };
 
