@@ -1,6 +1,6 @@
 #include <filezilla.h>
 #include "dialogex.h"
-#include <msgbox.h>
+#include "msgbox.h"
 
 BEGIN_EVENT_TABLE(wxDialogEx, wxDialog)
 EVT_CHAR_HOOK(wxDialogEx::OnChar)
@@ -37,10 +37,9 @@ bool wxDialogEx::ProcessEvent(wxEvent& event)
 
 void wxDialogEx::OnChar(wxKeyEvent& event)
 {
-	if (event.GetKeyCode() == WXK_ESCAPE)
-	{
-		wxCommandEvent event(wxEVT_COMMAND_BUTTON_CLICKED, wxID_CANCEL);
-		ProcessEvent(event);
+	if (event.GetKeyCode() == WXK_ESCAPE) {
+		wxCommandEvent cmdEvent(wxEVT_COMMAND_BUTTON_CLICKED, wxID_CANCEL);
+		ProcessEvent(cmdEvent);
 	}
 	else
 		event.Skip();
@@ -66,9 +65,9 @@ bool wxDialogEx::Load(wxWindow* pParent, const wxString& name)
 	return true;
 }
 
-bool wxDialogEx::SetChildLabel(int id, const wxString& label, unsigned long maxLength /*=0*/)
+bool wxDialogEx::SetChildLabel(int id, const wxString& label, unsigned long maxLength)
 {
-	wxStaticText* pText = wxDynamicCast(FindWindow(id), wxStaticText);
+	wxWindow* pText = FindWindow(id);
 	if (!pText)
 		return false;
 
@@ -83,9 +82,14 @@ bool wxDialogEx::SetChildLabel(int id, const wxString& label, unsigned long maxL
 	return true;
 }
 
+bool wxDialogEx::SetChildLabel(char const* id, const wxString& label, unsigned long maxLength)
+{
+	return SetChildLabel(XRCID(id), label, maxLength);
+}
+
 wxString wxDialogEx::GetChildLabel(int id)
 {
-	wxStaticText* pText = wxDynamicCast(FindWindow(id), wxStaticText);
+	auto pText = dynamic_cast<wxStaticText*>(FindWindow(id));
 	if (!pText)
 		return wxString();
 
@@ -126,17 +130,18 @@ bool wxDialogEx::ReplaceControl(wxWindow* old, wxWindow* wnd)
 
 bool wxDialogEx::CanShowPopupDialog()
 {
-	if( ShownDialogs() || IsShowingMessageBox() ) {
+	if (m_shown_dialogs != 0 || IsShowingMessageBox()) {
+		// There already is a dialog or message box showing
 		return false;
 	}
 
 	wxMouseState mouseState = wxGetMouseState();
-	if( mouseState.LeftIsDown() || mouseState.MiddleIsDown() || mouseState.RightIsDown() ) {
+	if (mouseState.LeftIsDown() || mouseState.MiddleIsDown() || mouseState.RightIsDown()) {
+		// Displaying a dialog while the user is clicking is extremely confusing, don't do it.
 		return false;
 	}
 #ifdef __WXMSW__
-	// Don't check for changes if mouse is captured,
-	// e.g. if user is dragging a file
+	// During a drag & drop we cannot show a dialog. Doing so can render the program unresponsive
 	if (GetCapture()) {
 		return false;
 	}
