@@ -17,7 +17,7 @@ class CMainFrame;
 class CPowerManagementInhibitor;
 #endif
 
-class CPowerManagement : protected CStateEventHandler
+class CPowerManagement : protected CGlobalStateEventHandler
 {
 public:
 	static void Create(CMainFrame* pMainFrame);
@@ -27,16 +27,16 @@ public:
 protected:
 	CPowerManagement(CMainFrame* pMainFrame);
 	virtual ~CPowerManagement();
-	
+
 	static CPowerManagement* m_pPowerManagement;
 
-	virtual void OnStateChange(CState* pState, enum t_statechange_notifications notification, const wxString& data, const void* data2);
+	virtual void OnStateChange(CState* pState, t_statechange_notifications notification, wxString const& data, void const* data2);
 
 	void DoSetBusy();
 	void DoSetIdle();
 
 	bool m_busy;
-	
+
 	CMainFrame* m_pMainFrame;
 
 #ifdef WITH_LIBDBUS
