@@ -290,8 +290,9 @@ bool CSiteManagerDialog::Create(wxWindow* parent, std::vector<_connected_site> *
 	if (m_pSiteManagerMutex->TryLock() == 0) {
 		int answer = wxMessageBoxEx(_("The Site Manager is opened in another instance of FileZilla 3.\nDo you want to continue? Any changes made in the Site Manager won't be saved then."),
 								  _("Site Manager already open"), wxYES_NO | wxICON_QUESTION);
-		if (answer != wxYES)
+		if (answer != wxYES) {
 			return false;
+		}
 
 		delete m_pSiteManagerMutex;
 		m_pSiteManagerMutex = 0;
@@ -300,16 +301,17 @@ bool CSiteManagerDialog::Create(wxWindow* parent, std::vector<_connected_site> *
 
 	// Now create the imagelist for the site tree
 	wxTreeCtrlEx *pTree = XRCCTRL(*this, "ID_SITETREE", wxTreeCtrlEx);
-	if (!pTree)
+	if (!pTree) {
 		return false;
+	}
 
 	wxSize s = CThemeProvider::GetIconSize(iconSizeSmall);
 	wxImageList* pImageList = new wxImageList(s.x, s.y);
 
-	pImageList->Add(wxArtProvider::GetBitmap(_T("ART_FOLDERCLOSED"), wxART_OTHER, s));
-	pImageList->Add(wxArtProvider::GetBitmap(_T("ART_FOLDER"), wxART_OTHER, s));
-	pImageList->Add(wxArtProvider::GetBitmap(_T("ART_SERVER"), wxART_OTHER, s));
-	pImageList->Add(wxArtProvider::GetBitmap(_T("ART_BOOKMARK"), wxART_OTHER, s));
+	pImageList->Add(CThemeProvider::Get()->CreateBitmap(_T("ART_FOLDERCLOSED"), wxART_OTHER, s));
+	pImageList->Add(CThemeProvider::Get()->CreateBitmap(_T("ART_FOLDER"), wxART_OTHER, s));
+	pImageList->Add(CThemeProvider::Get()->CreateBitmap(_T("ART_SERVER"), wxART_OTHER, s));
+	pImageList->Add(CThemeProvider::Get()->CreateBitmap(_T("ART_BOOKMARK"), wxART_OTHER, s));
 
 	pTree->AssignImageList(pImageList);
 
@@ -847,7 +849,7 @@ bool CSiteManagerDialog::SaveChild(pugi::xml_node element, wxTreeItemId child)
 		return false;
 	}
 
-	wxString const name = pTree->GetItemText(child);
+	std::wstring const name = pTree->GetItemText(child).ToStdWstring();
 
 	CSiteManagerItemData* data = static_cast<CSiteManagerItemData* >(pTree->GetItemData(child));
 	if (!data) {
@@ -862,19 +864,19 @@ bool CSiteManagerDialog::SaveChild(pugi::xml_node element, wxTreeItemId child)
 		SetServer(node, data->m_site->m_server);
 
 		// Save comments
-		AddTextElement(node, "Comments", data->m_site->m_comments);
+		AddTextElement(node, "Comments", data->m_site->m_comments.ToStdWstring());
 
 		// Save colour
 		AddTextElement(node, "Colour", CSiteManager::GetColourIndex(data->m_site->m_colour));
 
 		// Save local dir
-		AddTextElement(node, "LocalDir", data->m_site->m_default_bookmark.m_localDir);
+		AddTextElement(node, "LocalDir", data->m_site->m_default_bookmark.m_localDir.ToStdWstring());
 
 		// Save remote dir
 		AddTextElement(node, "RemoteDir", data->m_site->m_default_bookmark.m_remoteDir.GetSafePath());
 
-		AddTextElementRaw(node, "SyncBrowsing", data->m_site->m_default_bookmark.m_sync ? "1" : "0");
-		AddTextElementRaw(node, "DirectoryComparison", data->m_site->m_default_bookmark.m_comparison ? "1" : "0");
+		AddTextElementUtf8(node, "SyncBrowsing", data->m_site->m_default_bookmark.m_sync ? "1" : "0");
+		AddTextElementUtf8(node, "DirectoryComparison", data->m_site->m_default_bookmark.m_comparison ? "1" : "0");
 		AddTextElement(node, name);
 
 		data->m_site->m_bookmarks.clear();
@@ -900,13 +902,13 @@ bool CSiteManagerDialog::SaveChild(pugi::xml_node element, wxTreeItemId child)
 		AddTextElement(node, "Name", name);
 
 		// Save local dir
-		AddTextElement(node, "LocalDir", data->m_bookmark->m_localDir);
+		AddTextElement(node, "LocalDir", data->m_bookmark->m_localDir.ToStdWstring());
 
 		// Save remote dir
 		AddTextElement(node, "RemoteDir", data->m_bookmark->m_remoteDir.GetSafePath());
 
-		AddTextElementRaw(node, "SyncBrowsing", data->m_bookmark->m_sync ? "1" : "0");
-		AddTextElementRaw(node, "DirectoryComparison", data->m_bookmark->m_comparison ? "1" : "0");
+		AddTextElementUtf8(node, "SyncBrowsing", data->m_bookmark->m_sync ? "1" : "0");
+		AddTextElementUtf8(node, "DirectoryComparison", data->m_bookmark->m_comparison ? "1" : "0");
 	}
 
 	return true;
@@ -915,12 +917,14 @@ bool CSiteManagerDialog::SaveChild(pugi::xml_node element, wxTreeItemId child)
 void CSiteManagerDialog::OnNewFolder(wxCommandEvent&)
 {
 	wxTreeCtrlEx *pTree = XRCCTRL(*this, "ID_SITETREE", wxTreeCtrlEx);
-	if (!pTree)
+	if (!pTree) {
 		return;
+	}
 
 	wxTreeItemId item = pTree->GetSelection();
-	if (!item.IsOk())
+	if (!item.IsOk()) {
 		return;
+	}
 
 	while (pTree->GetItemData(item))
 		item = pTree->GetItemParent(item);
@@ -2255,8 +2259,9 @@ void CSiteManagerDialog::RememberLastSelected()
 void CSiteManagerDialog::OnContextMenu(wxTreeEvent& event)
 {
 	wxMenu* pMenu = wxXmlResource::Get()->LoadMenu(_T("ID_MENU_SITEMANAGER"));
-	if (!pMenu)
+	if (!pMenu) {
 		return;
+	}
 
 	m_contextMenuItem = event.GetItem();
 
@@ -2275,10 +2280,11 @@ void CSiteManagerDialog::OnExportSelected(wxCommandEvent&)
 					_T("sites.xml"), _T("XML files (*.xml)|*.xml"),
 					wxFD_SAVE | wxFD_OVERWRITE_PROMPT);
 
-	if (dlg.ShowModal() != wxID_OK)
+	if (dlg.ShowModal() != wxID_OK) {
 		return;
+	}
 
-	CXmlFile xml(dlg.GetPath());
+	CXmlFile xml(dlg.GetPath().ToStdWstring());
 
 	auto exportRoot = xml.CreateEmpty();
 
