@@ -7,7 +7,7 @@
 
 #include <libfilezilla/local_filesys.hpp>
 
-CXmlFile::CXmlFile(wxString const& fileName, wxString const& root)
+CXmlFile::CXmlFile(std::wstring const& fileName, std::string const& root)
 {
 	if (!root.empty()) {
 		m_rootName = root;
@@ -15,26 +15,21 @@ CXmlFile::CXmlFile(wxString const& fileName, wxString const& root)
 	SetFileName(fileName);
 }
 
-void CXmlFile::SetFileName(const wxString& name)
+void CXmlFile::SetFileName(std::wstring const& name)
 {
-	wxASSERT(!name.empty());
+	assert(!name.empty());
 	m_fileName = name;
 	m_modificationTime = fz::datetime();
 }
 
-CXmlFile::~CXmlFile()
-{
-	Close();
-}
-
-pugi::xml_node CXmlFile::Load()
+pugi::xml_node CXmlFile::Load(bool overwriteInvalid)
 {
 	Close();
 	m_error.clear();
 
 	wxCHECK(!m_fileName.empty(), m_element);
 
-	wxString redirectedName = GetRedirectedName();
+	std::wstring redirectedName = GetRedirectedName();
 
 	GetXmlFile(redirectedName);
 	if (!m_element) {
@@ -49,8 +44,17 @@ pugi::xml_node CXmlFile::Load()
 		// Try the backup file
 		GetXmlFile(redirectedName + _T("~"));
 		if (!m_element) {
-			// Loading backup failed. If both original and backup file are empty, create new file.
+			// Loading backup failed.
+
+			// Create new one if we are allowed to create empty file
+			bool createEmpty = overwriteInvalid;
+
+			// Also, if both original and backup file are empty, create new file.
 			if (fz::local_filesys::get_size(fz::to_native(redirectedName)) <= 0 && fz::local_filesys::get_size(fz::to_native(redirectedName + _T("~"))) <= 0) {
+				createEmpty = true;
+			}
+
+			if (createEmpty) {
 				m_error.clear();
 				CreateEmpty();
 				m_modificationTime = fz::local_filesys::get_modification_time(fz::to_native(redirectedName));
@@ -73,8 +77,8 @@ pugi::xml_node CXmlFile::Load()
 		if (!res) {
 			// Could not restore backup, give up.
 			Close();
-			m_error = err;
-			m_error += _T("\n") + wxString::Format(_("The valid backup file %s could not be restored"), redirectedName + _T("~"));
+			m_error = err.ToStdWstring();
+			m_error += _T("\n") + wxString::Format(_("The valid backup file %s could not be restored"), redirectedName + _T("~")).ToStdWstring();
 			m_modificationTime.clear();
 			return m_element;
 		}
@@ -116,15 +120,15 @@ void CXmlFile::UpdateMetadata()
 
 	SetTextAttribute(m_element, "version", CBuildInfo::GetVersion());
 
-	wxString const platform =
-#ifdef __WXMSW__
-		_T("windows");
-#elif defined(__WXMAX__)
-		_T("mac");
+	std::string const platform =
+#ifdef FZ_WINDOWS
+		"windows";
+#elif defined(FZ_MAC)
+		"mac";
 #else
-		_T("*nix");
+		"*nix";
 #endif
-	SetTextAttribute(m_element, "platform", platform);
+	SetTextAttributeUtf8(m_element, "platform", platform);
 }
 
 bool CXmlFile::Save(bool printError)
@@ -140,7 +144,7 @@ bool CXmlFile::Save(bool printError)
 	m_modificationTime = fz::local_filesys::get_modification_time(fz::to_native(m_fileName));
 
 	if (!res && printError) {
-		wxASSERT(!m_error.empty());
+		assert(!m_error.empty());
 
 		wxString msg = wxString::Format(_("Could not write \"%s\":"), m_fileName);
 		wxMessageBoxEx(msg + _T("\n") + m_error, _("Error writing xml file"), wxICON_ERROR);
@@ -156,18 +160,18 @@ pugi::xml_node CXmlFile::CreateEmpty()
 	decl.append_attribute("version") = "1.0";
 	decl.append_attribute("encoding") = "UTF-8";
 
-	m_element = m_document.append_child(m_rootName.utf8_str());
+	m_element = m_document.append_child(m_rootName.c_str());
 	return m_element;
 }
 
-wxString ConvLocal(const char *value)
+pugi::xml_node AddTextElement(pugi::xml_node node, const char* name, std::string const& value, bool overwrite)
 {
-	return wxString(wxConvUTF8.cMB2WC(value), *wxConvCurrent);
+	return AddTextElementUtf8(node, name, fz::to_utf8(value), overwrite);
 }
 
-pugi::xml_node AddTextElement(pugi::xml_node node, const char* name, const wxString& value, bool overwrite)
+pugi::xml_node AddTextElement(pugi::xml_node node, const char* name, std::wstring const& value, bool overwrite)
 {
-	return AddTextElementRaw(node, name, fz::to_utf8(value).c_str(), overwrite);
+	return AddTextElementUtf8(node, name, fz::to_utf8(value), overwrite);
 }
 
 void AddTextElement(pugi::xml_node node, const char* name, int64_t value, bool overwrite)
@@ -179,50 +183,42 @@ void AddTextElement(pugi::xml_node node, const char* name, int64_t value, bool o
 	child.text().set(static_cast<long long>(value));
 }
 
-pugi::xml_node AddTextElementRaw(pugi::xml_node node, const char* name, const char* value, bool overwrite)
+pugi::xml_node AddTextElementUtf8(pugi::xml_node node, const char* name, std::string const& value, bool overwrite)
 {
-	wxASSERT(node);
-	wxASSERT(value);
+	assert(node);
 
 	if (overwrite) {
 		node.remove_child(name);
 	}
 
 	auto element = node.append_child(name);
-	if (*value)
-		element.text().set(value);
+	if (!value.empty()) {
+		element.text().set(value.c_str());
+	}
 
 	return element;
 }
 
-void AddTextElement(pugi::xml_node node, const wxString& value)
+void AddTextElement(pugi::xml_node node, std::string const& value)
 {
-	wxASSERT(node);
-	wxASSERT(value);
+	AddTextElementUtf8(node, fz::to_utf8(value));
+}
 
-	wxScopedCharBuffer utf8 = value.utf8_str();
-	if (!utf8)
-		return;
-
-	AddTextElementRaw(node, utf8);
+void AddTextElement(pugi::xml_node node, std::wstring const& value)
+{
+	AddTextElementUtf8(node, fz::to_utf8(value));
 }
 
 void AddTextElement(pugi::xml_node node, int64_t value)
 {
-	wxASSERT(node);
+	assert(node);
 	node.text().set(static_cast<long long>(value));
 }
 
-void AddTextElementRaw(pugi::xml_node node, const char* value)
+void AddTextElementUtf8(pugi::xml_node node, std::string const& value)
 {
-	wxASSERT(node);
-	wxASSERT(value);
-
-	if (*value)
-		node.text().set(value);
-	else {
-		node.text().set("");
-	}
+	assert(node);
+	node.text().set(value.c_str());
 }
 
 std::wstring GetTextElement_Trimmed(pugi::xml_node node, const char* name)
@@ -262,8 +258,8 @@ bool GetTextElementBool(pugi::xml_node node, const char* name, bool defValue /*=
 }
 
 // Opens the specified XML file if it exists or creates a new one otherwise.
-// Returns 0 on error.
-bool CXmlFile::GetXmlFile(wxString const& file)
+// Returns false on error.
+bool CXmlFile::GetXmlFile(std::wstring const& file)
 {
 	Close();
 
@@ -274,34 +270,34 @@ bool CXmlFile::GetXmlFile(wxString const& file)
 	// File exists, open it
 	auto result = m_document.load_file(static_cast<wchar_t const*>(file.c_str()));
 	if (!result) {
-		m_error += wxString::Format(_T("%s at offset %lld."), wxString(result.description()), static_cast<long long>(result.offset));
+		m_error += fz::sprintf(_T("%s at offset %d."), result.description(), result.offset);
 		return false;
 	}
 
-	m_element = m_document.child(m_rootName.utf8_str());
+	m_element = m_document.child(m_rootName.c_str());
 	if (!m_element) {
 		if (m_document.first_child()) { // Beware: parse_declaration and parse_doctype can break this
 			// Not created by FileZilla3
 			Close();
-			m_error = _("Unknown root element, the file does not appear to be generated by FileZilla.");
+			m_error = _("Unknown root element, the file does not appear to be generated by FileZilla.").ToStdWstring();
 			return false;
 		}
-		m_element = m_document.append_child(m_rootName.utf8_str());
+		m_element = m_document.append_child(m_rootName.c_str());
 	}
 
 	return true;
 }
 
-wxString CXmlFile::GetRedirectedName() const
+std::wstring CXmlFile::GetRedirectedName() const
 {
-	wxString redirectedName = m_fileName;
+	std::wstring redirectedName = m_fileName;
 	bool isLink = false;
 	if (fz::local_filesys::get_file_info(fz::to_native(redirectedName), isLink, 0, 0, 0) == fz::local_filesys::file) {
 		if (isLink) {
 			CLocalPath target(fz::to_wstring(fz::local_filesys::get_link_target(fz::to_native(redirectedName))));
 			if (!target.empty()) {
 				redirectedName = target.GetPath();
-				redirectedName.RemoveLast();
+				redirectedName.pop_back();
 			}
 		}
 	}
@@ -329,7 +325,7 @@ bool CXmlFile::SaveXmlFile()
 			res = wxCopyFile(redirectedName, redirectedName + _T("~"));
 		}
 		if (!res) {
-			m_error = _("Failed to create backup copy of xml file");
+			m_error = _("Failed to create backup copy of xml file").ToStdWstring();
 			return false;
 		}
 	}
@@ -342,7 +338,7 @@ bool CXmlFile::SaveXmlFile()
 			wxLogNull null;
 			wxRenameFile(redirectedName + _T("~"), redirectedName);
 		}
-		m_error = _("Failed to write xml file");
+		m_error = _("Failed to write xml file").ToStdWstring();
 		return false;
 	}
 
@@ -527,7 +523,7 @@ void SetServer(pugi::xml_node node, const CServer& server)
 			else {
 				std::string pass = fz::to_utf8(server.GetPass());
 				
-				pugi::xml_node passElement = AddTextElementRaw(node, "Pass", fz::base64_encode(pass).c_str());
+				pugi::xml_node passElement = AddTextElementUtf8(node, "Pass", fz::base64_encode(pass));
 				if (passElement) {
 					SetTextAttribute(passElement, "encoding", _T("base64"));
 				}
@@ -547,13 +543,13 @@ void SetServer(pugi::xml_node node, const CServer& server)
 	switch (server.GetPasvMode())
 	{
 	case MODE_PASSIVE:
-		AddTextElementRaw(node, "PasvMode", "MODE_PASSIVE");
+		AddTextElementUtf8(node, "PasvMode", "MODE_PASSIVE");
 		break;
 	case MODE_ACTIVE:
-		AddTextElementRaw(node, "PasvMode", "MODE_ACTIVE");
+		AddTextElementUtf8(node, "PasvMode", "MODE_ACTIVE");
 		break;
 	default:
-		AddTextElementRaw(node, "PasvMode", "MODE_DEFAULT");
+		AddTextElementUtf8(node, "PasvMode", "MODE_DEFAULT");
 		break;
 	}
 	AddTextElement(node, "MaximumMultipleConnections", server.MaximumMultipleConnections());
@@ -561,13 +557,13 @@ void SetServer(pugi::xml_node node, const CServer& server)
 	switch (server.GetEncodingType())
 	{
 	case ENCODING_AUTO:
-		AddTextElementRaw(node, "EncodingType", "Auto");
+		AddTextElementUtf8(node, "EncodingType", "Auto");
 		break;
 	case ENCODING_UTF8:
-		AddTextElementRaw(node, "EncodingType", "UTF-8");
+		AddTextElementUtf8(node, "EncodingType", "UTF-8");
 		break;
 	case ENCODING_CUSTOM:
-		AddTextElementRaw(node, "EncodingType", "Custom");
+		AddTextElementUtf8(node, "EncodingType", "Custom");
 		AddTextElement(node, "CustomEncoding", server.GetCustomEncoding());
 		break;
 	}
@@ -582,31 +578,36 @@ void SetServer(pugi::xml_node node, const CServer& server)
 		}
 	}
 
-	AddTextElementRaw(node, "BypassProxy", server.GetBypassProxy() ? "1" : "0");
+	AddTextElementUtf8(node, "BypassProxy", server.GetBypassProxy() ? "1" : "0");
 	std::wstring const& name = server.GetName();
 	if (!name.empty()) {
 		AddTextElement(node, "Name", name);
 	}
 }
 
-void SetTextAttribute(pugi::xml_node node, const char* name, const wxString& value)
+void SetTextAttribute(pugi::xml_node node, char const* name, std::string const& value)
 {
-	wxASSERT(node);
+	SetTextAttributeUtf8(node, name, fz::to_utf8(value));
+}
 
-	wxScopedCharBuffer utf8 = value.utf8_str();
-	if (!utf8)
-		return;
+void SetTextAttribute(pugi::xml_node node, char const* name, std::wstring const& value)
+{
+	SetTextAttributeUtf8(node, name, fz::to_utf8(value));
+}
 
+void SetTextAttributeUtf8(pugi::xml_node node, char const* name, std::string const& utf8)
+{
+	assert(node);
 	auto attribute = node.attribute(name);
 	if (!attribute) {
 		attribute = node.append_attribute(name);
 	}
-	attribute.set_value(utf8);
+	attribute.set_value(utf8.c_str());
 }
 
-std::wstring GetTextAttribute(pugi::xml_node node, const char* name)
+std::wstring GetTextAttribute(pugi::xml_node node, char const* name)
 {
-	wxASSERT(node);
+	assert(node);
 
 	const char* value = node.attribute(name).value();
 	return fz::to_wstring_from_utf8(value);
@@ -617,8 +618,9 @@ pugi::xml_node FindElementWithAttribute(pugi::xml_node node, const char* element
 	pugi::xml_node child = element ? node.child(element) : node.first_child();
 	while (child) {
 		const char* nodeVal = child.attribute(attribute).value();
-		if (nodeVal && !strcmp(value, nodeVal))
+		if (nodeVal && !strcmp(value, nodeVal)) {
 			return child;
+		}
 
 		child = element ? child.next_sibling(element) : child.next_sibling();
 	}
@@ -684,7 +686,7 @@ bool CXmlFile::ParseData(char* data)
 {
 	Close();
 	m_document.load_string(data);
-	m_element = m_document.child(m_rootName.mb_str());
+	m_element = m_document.child(m_rootName.c_str());
 	if (!m_element) {
 		Close();
 	}
@@ -696,6 +698,6 @@ bool CXmlFile::IsFromFutureVersion() const
 	if (!m_element) {
 		return false;
 	}
-	wxString const version = GetTextAttribute(m_element, "version");
+	std::wstring const version = GetTextAttribute(m_element, "version");
 	return CBuildInfo::ConvertToVersionNumber(CBuildInfo::GetVersion().c_str()) < CBuildInfo::ConvertToVersionNumber(version.c_str());
 }
