@@ -1,9 +1,12 @@
 #include <libfilezilla_engine.h>
 #include <directorylistingparser.h>
 
+#include <libfilezilla/format.hpp>
+
 #include <cppunit/extensions/HelperMacros.h>
 #include <list>
 
+#include <string.h>
 /*
  * This testsuite asserts the correctness of the directory listing parser.
  * It's main purpose is to ensure that all known formats are recognized and
@@ -82,10 +85,10 @@ void CDirectoryListingParserTest::InitEntries()
 	m_entries.emplace_back(t_entry({
 			"dr-xr-xr-x   2 root     other        512 Apr  8  1994 01-unix-std dir",
 			{
-				_T("01-unix-std dir"),
+				L"01-unix-std dir",
 				512,
-				R(_T("dr-xr-xr-x")),
-				R(_T("root other")),
+				R(L"dr-xr-xr-x"),
+				R(L"root other"),
 				CDirentry::flag_dir,
 				O(),
 				fz::datetime(fz::datetime::utc, 1994, 4, 8)
@@ -97,10 +100,10 @@ void CDirectoryListingParserTest::InitEntries()
 	m_entries.emplace_back(t_entry({
 			"-rw-r--r--   1 root     other        531 3 29 03:26 02-unix-std file",
 			{
-				_T("02-unix-std file"),
+				L"02-unix-std file",
 				531,
-				R(_T("-rw-r--r--")),
-				R(_T("root other")),
+				R(L"-rw-r--r--"),
+				R(L"root other"),
 				0,
 				O(),
 				fz::datetime(fz::datetime::utc, calcYear(3, 29), 3, 29, 3, 26)
@@ -112,10 +115,10 @@ void CDirectoryListingParserTest::InitEntries()
 	m_entries.emplace_back(t_entry({
 			"dr-xr-xr-x   2 root                  512 Apr  8  1994 03-unix-nogroup dir",
 			{
-				_T("03-unix-nogroup dir"),
+				L"03-unix-nogroup dir",
 				512,
-				R(_T("dr-xr-xr-x")),
-				R(_T("root")),
+				R(L"dr-xr-xr-x"),
+				R(L"root"),
 				CDirentry::flag_dir,
 				O(),
 				fz::datetime(fz::datetime::utc, 1994, 4, 8)
@@ -127,12 +130,12 @@ void CDirectoryListingParserTest::InitEntries()
 	m_entries.emplace_back(t_entry({
 			"lrwxrwxrwx   1 root     other          7 Jan 25 00:17 04-unix-std link -> usr/bin",
 			{
-				_T("04-unix-std link"),
+				L"04-unix-std link",
 				7,
-				R(_T("lrwxrwxrwx")),
-				R(_T("root other")),
+				R(L"lrwxrwxrwx"),
+				R(L"root other"),
 				CDirentry::flag_dir | CDirentry::flag_link,
-				O(_T("usr/bin")),
+				O(L"usr/bin"),
 				fz::datetime(fz::datetime::utc, calcYear(1, 25), 1, 25, 0, 17)
 			},
 			DEFAULT
@@ -144,10 +147,10 @@ void CDirectoryListingParserTest::InitEntries()
 	m_entries.emplace_back(t_entry({
 			"-rw-r--r--   1 root     other        531 09-26 2000 05-unix-date file",
 			{
-				_T("05-unix-date file"),
+				L"05-unix-date file",
 				531,
-				R(_T("-rw-r--r--")),
-				R(_T("root other")),
+				R(L"-rw-r--r--"),
+				R(L"root other"),
 				0,
 				O(),
 				fz::datetime(fz::datetime::utc, 2000, 9, 26)
@@ -158,10 +161,10 @@ void CDirectoryListingParserTest::InitEntries()
 	m_entries.emplace_back(t_entry({
 			"-rw-r--r--   1 root     other        531 09-26 13:45 06-unix-date file",
 			{
-				_T("06-unix-date file"),
+				L"06-unix-date file",
 				531,
-				R(_T("-rw-r--r--")),
-				R(_T("root other")),
+				R(L"-rw-r--r--"),
+				R(L"root other"),
 				0,
 				O(),
 				fz::datetime(fz::datetime::utc, calcYear(9, 26), 9, 26, 13, 45)
@@ -172,10 +175,10 @@ void CDirectoryListingParserTest::InitEntries()
 	m_entries.emplace_back(t_entry({
 			"-rw-r--r--   1 root     other        531 2005-06-07 21:22 07-unix-date file",
 			{
-				_T("07-unix-date file"),
+				L"07-unix-date file",
 				531,
-				R(_T("-rw-r--r--")),
-				R(_T("root other")),
+				R(L"-rw-r--r--"),
+				R(L"root other"),
 				0,
 				O(),
 				fz::datetime(fz::datetime::utc, 2005, 6, 7, 21, 22)
@@ -188,10 +191,10 @@ void CDirectoryListingParserTest::InitEntries()
 	m_entries.emplace_back(t_entry({
 			"-rw-r--r--   1 root     other  33.5k Oct 5 21:22 08-unix-namedsize file",
 			{
-				_T("08-unix-namedsize file"),
+				L"08-unix-namedsize file",
 				335 * 1024 / 10,
-				R(_T("-rw-r--r--")),
-				R(_T("root other")),
+				R(L"-rw-r--r--"),
+				R(L"root other"),
 				0,
 				O(),
 				fz::datetime(fz::datetime::utc, calcYear(10, 5), 10, 5, 21, 22)
@@ -205,10 +208,10 @@ void CDirectoryListingParserTest::InitEntries()
 	m_entries.emplace_back(t_entry({
 			"d [R----F--] supervisor            512       Jan 16 18:53    09-netware dir",
 			{
-				_T("09-netware dir"),
+				L"09-netware dir",
 				512,
-				R(_T("d [R----F--]")),
-				R(_T("supervisor")),
+				R(L"d [R----F--]"),
+				R(L"supervisor"),
 				CDirentry::flag_dir,
 				O(),
 				fz::datetime(fz::datetime::utc, calcYear(1, 16), 1, 16, 18, 53)
@@ -219,10 +222,10 @@ void CDirectoryListingParserTest::InitEntries()
 	m_entries.emplace_back(t_entry({
 			"- [R----F--] rhesus             214059       Oct 20 15:27    10-netware file",
 			{
-				_T("10-netware file"),
+				L"10-netware file",
 				214059,
-				R(_T("- [R----F--]")),
-				R(_T("rhesus")),
+				R(L"- [R----F--]"),
+				R(L"rhesus"),
 				0,
 				O(),
 				fz::datetime(fz::datetime::utc, calcYear(10, 20), 10, 20, 15, 27)
@@ -240,10 +243,10 @@ void CDirectoryListingParserTest::InitEntries()
 	m_entries.emplace_back(t_entry({
 			"-------r--         326  1391972  1392298 Nov 22  1995 11-netpresenz file",
 			{
-				_T("11-netpresenz file"),
+				L"11-netpresenz file",
 				1392298,
-				R(_T("-------r--")),
-				R(_T("1391972")),
+				R(L"-------r--"),
+				R(L"1391972"),
 				0,
 				O(),
 				fz::datetime(fz::datetime::utc, 1995, 11, 22)
@@ -254,10 +257,10 @@ void CDirectoryListingParserTest::InitEntries()
 	m_entries.emplace_back(t_entry({
 			"drwxrwxr-x               folder        2 May 10  1996 12-netpresenz dir",
 			{
-				_T("12-netpresenz dir"),
+				L"12-netpresenz dir",
 				2,
-				R(_T("drwxrwxr-x")),
-				R(_T("folder")),
+				R(L"drwxrwxr-x"),
+				R(L"folder"),
 				CDirentry::flag_dir,
 				O(),
 				fz::datetime(fz::datetime::utc, 1996, 5, 10)
@@ -269,10 +272,10 @@ void CDirectoryListingParserTest::InitEntries()
 	m_entries.emplace_back(t_entry({
 			"-rw-r--r--   1 group domain user 531 Jan 29 03:26 13-unix-domain file",
 			{
-				_T("13-unix-domain file"),
+				L"13-unix-domain file",
 				531,
-				R(_T("-rw-r--r--")),
-				R(_T("group domain user")),
+				R(L"-rw-r--r--"),
+				R(L"group domain user"),
 				0,
 				O(),
 				fz::datetime(fz::datetime::utc, calcYear(1, 29), 1, 29, 3, 26)
@@ -288,9 +291,9 @@ void CDirectoryListingParserTest::InitEntries()
 	m_entries.emplace_back(t_entry({
 			"+i8388621.48594,m825718503,r,s280,up755\t14-eplf file",
 			{
-				_T("14-eplf file"),
+				L"14-eplf file",
 				280,
-				R(_T("755")),
+				R(L"755"),
 				R(),
 				0,
 				O(),
@@ -302,7 +305,7 @@ void CDirectoryListingParserTest::InitEntries()
 	m_entries.emplace_back(t_entry({
 			"+i8388621.50690,m824255907,/,\t15-eplf dir",
 			{
-				_T("15-eplf dir"),
+				L"15-eplf dir",
 				-1,
 				R(),
 				R(),
@@ -319,7 +322,7 @@ void CDirectoryListingParserTest::InitEntries()
 	m_entries.emplace_back(t_entry({
 			"04-27-00  12:09PM       <DIR>          16-dos-dateambiguous dir",
 			{
-				_T("16-dos-dateambiguous dir"),
+				L"16-dos-dateambiguous dir",
 				-1,
 				R(),
 				R(),
@@ -334,7 +337,7 @@ void CDirectoryListingParserTest::InitEntries()
 	m_entries.emplace_back(t_entry({
 			"04-06-00  03:47PM                  589 17-dos-dateambiguous file",
 			{
-				_T("17-dos-dateambiguous file"),
+				L"17-dos-dateambiguous file",
 				589,
 				R(),
 				R(),
@@ -348,7 +351,7 @@ void CDirectoryListingParserTest::InitEntries()
 	m_entries.emplace_back(t_entry({
 			"2002-09-02  18:48       <DIR>          18-dos-longyear dir",
 			{
-				_T("18-dos-longyear dir"),
+				L"18-dos-longyear dir",
 				-1,
 				R(),
 				R(),
@@ -362,7 +365,7 @@ void CDirectoryListingParserTest::InitEntries()
 	m_entries.emplace_back(t_entry({
 			"2002-09-02  19:06                9,730 19-dos-longyear file",
 			{
-				_T("19-dos-longyear file"),
+				L"19-dos-longyear file",
 				9730,
 				R(),
 				R(),
@@ -377,10 +380,10 @@ void CDirectoryListingParserTest::InitEntries()
 	m_entries.emplace_back(t_entry({
 			"0100644   500  101   12345    123456789       20-unix-numerical file",
 			{
-				_T("20-unix-numerical file"),
+				L"20-unix-numerical file",
 				12345,
-				R(_T("0100644")),
-				R(_T("500 101")),
+				R(L"0100644"),
+				R(L"500 101"),
 				0,
 				O(),
 				fz::datetime(fz::datetime::utc, 1973, 11, 29, 21, 33, 9)
@@ -394,7 +397,7 @@ void CDirectoryListingParserTest::InitEntries()
 	m_entries.emplace_back(t_entry({
 			"206876  Apr 04, 2000 21:06 21-vshell-old file",
 			{
-				_T("21-vshell-old file"),
+				L"21-vshell-old file",
 				206876,
 				R(),
 				R(),
@@ -408,7 +411,7 @@ void CDirectoryListingParserTest::InitEntries()
 	m_entries.emplace_back(t_entry({
 			"0  Dec 12, 2002 02:13 22-vshell-old dir/",
 			{
-				_T("22-vshell-old dir"),
+				L"22-vshell-old dir",
 				0,
 				R(),
 				R(),
@@ -424,10 +427,10 @@ void CDirectoryListingParserTest::InitEntries()
 	m_entries.emplace_back(t_entry({
 			"-rwxr-xr-x    1 user group        9 Oct 08, 2002 09:47 23-vshell-new file",
 			{
-				_T("23-vshell-new file"),
+				L"23-vshell-new file",
 				9,
-				R(_T("-rwxr-xr-x")),
-				R(_T("user group")),
+				R(L"-rwxr-xr-x"),
+				R(L"user group"),
 				0,
 				O(),
 				fz::datetime(fz::datetime::utc, 2002, 10, 8, 9, 47)
@@ -442,7 +445,7 @@ void CDirectoryListingParserTest::InitEntries()
 	m_entries.emplace_back(t_entry({
 			"36611      A    04-23-103  10:57  24-os2 file",
 			{
-				_T("24-os2 file"),
+				L"24-os2 file",
 				36611,
 				R(),
 				R(),
@@ -456,7 +459,7 @@ void CDirectoryListingParserTest::InitEntries()
 	m_entries.emplace_back(t_entry({
 			" 1123      A    07-14-99   12:37  25-os2 file",
 			{
-				_T("25-os2 file"),
+				L"25-os2 file",
 				1123,
 				R(),
 				R(),
@@ -471,7 +474,7 @@ void CDirectoryListingParserTest::InitEntries()
 	m_entries.emplace_back(t_entry({
 			"    0 DIR       02-11-103  16:15  26-os2 dir",
 			{
-				_T("26-os2 dir"),
+				L"26-os2 dir",
 				0,
 				R(),
 				R(),
@@ -486,7 +489,7 @@ void CDirectoryListingParserTest::InitEntries()
 	m_entries.emplace_back(t_entry({
 			" 1123 DIR  A    10-05-100  23:38  27-os2 dir",
 			{
-				_T("27-os2 dir"),
+				L"27-os2 dir",
 				1123,
 				R(),
 				R(),
@@ -503,10 +506,10 @@ void CDirectoryListingParserTest::InitEntries()
 	m_entries.emplace_back(t_entry({
 			"dr-xr-xr-x   2 root     other      2235 26. Juli, 20:10 28-datetest-ger dir",
 			{
-				_T("28-datetest-ger dir"),
+				L"28-datetest-ger dir",
 				2235,
-				R(_T("dr-xr-xr-x")),
-				R(_T("root other")),
+				R(L"dr-xr-xr-x"),
+				R(L"root other"),
 				CDirentry::flag_dir,
 				O(),
 				fz::datetime(fz::datetime::utc, calcYear(7, 26), 7, 26, 20, 10)
@@ -517,10 +520,10 @@ void CDirectoryListingParserTest::InitEntries()
 	m_entries.emplace_back(t_entry({
 			"dr-xr-xr-x   2 root     other      2235 szept 26 20:10 28b-datetest-hungarian dir",
 			{
-				_T("28b-datetest-hungarian dir"),
+				L"28b-datetest-hungarian dir",
 				2235,
-				R(_T("dr-xr-xr-x")),
-				R(_T("root other")),
+				R(L"dr-xr-xr-x"),
+				R(L"root other"),
 				CDirentry::flag_dir,
 				O(),
 				fz::datetime(fz::datetime::utc, calcYear(9, 26), 9, 26, 20, 10)
@@ -531,10 +534,10 @@ void CDirectoryListingParserTest::InitEntries()
 	m_entries.emplace_back(t_entry({
 			"-r-xr-xr-x   2 root     other      2235 2.   Okt.  2003 29-datetest-ger file",
 			{
-				_T("29-datetest-ger file"),
+				L"29-datetest-ger file",
 				2235,
-				R(_T("-r-xr-xr-x")),
-				R(_T("root other")),
+				R(L"-r-xr-xr-x"),
+				R(L"root other"),
 				0,
 				O(),
 				fz::datetime(fz::datetime::utc, 2003, 10, 2)
@@ -545,10 +548,10 @@ void CDirectoryListingParserTest::InitEntries()
 	m_entries.emplace_back(t_entry({
 			"-r-xr-xr-x   2 root     other      2235 1999/10/12 17:12 30-datetest file",
 			{
-				_T("30-datetest file"),
+				L"30-datetest file",
 				2235,
-				R(_T("-r-xr-xr-x")),
-				R(_T("root other")),
+				R(L"-r-xr-xr-x"),
+				R(L"root other"),
 				0,
 				O(),
 				fz::datetime(fz::datetime::utc, 1999, 10, 12, 17, 12)
@@ -559,10 +562,10 @@ void CDirectoryListingParserTest::InitEntries()
 	m_entries.emplace_back(t_entry({
 			"-r-xr-xr-x   2 root     other      2235 24-04-2003 17:12 31-datetest file",
 			{
-				_T("31-datetest file"),
+				L"31-datetest file",
 				2235,
-				R(_T("-r-xr-xr-x")),
-				R(_T("root other")),
+				R(L"-r-xr-xr-x"),
+				R(L"root other"),
 				0,
 				O(),
 				fz::datetime(fz::datetime::utc, 2003, 4, 24, 17, 12)
@@ -576,10 +579,10 @@ void CDirectoryListingParserTest::InitEntries()
 	m_entries.emplace_back(t_entry({
 			"-rw-r--r--   1 root       sys           8473  4\x8c\x8e 18\x93\xfa 2003\x94\x4e 32-datatest-japanese file",
 			{
-				_T("32-datatest-japanese file"),
+				L"32-datatest-japanese file",
 				8473,
-				R(_T("-rw-r--r--")),
-				R(_T("root sys")),
+				R(L"-rw-r--r--"),
+				R(L"root sys"),
 				0,
 				O(),
 				fz::datetime(fz::datetime::utc, 2003, 4, 18)
@@ -592,10 +595,10 @@ void CDirectoryListingParserTest::InitEntries()
 	m_entries.emplace_back(t_entry({
 			"-rwxrwxrwx   1 root     staff          0 2003   3\xed\xef 20 33-asian date file",
 			{
-				_T("33-asian date file"),
+				L"33-asian date file",
 				0,
-				R(_T("-rwxrwxrwx")),
-				R(_T("root staff")),
+				R(L"-rwxrwxrwx"),
+				R(L"root staff"),
 				0,
 				O(),
 				fz::datetime(fz::datetime::utc, 2003, 3, 20)
@@ -606,10 +609,10 @@ void CDirectoryListingParserTest::InitEntries()
 	m_entries.emplace_back(t_entry({
 			"-r--r--r-- 1 root root 2096 8\xed 17 08:52 34-asian date file",
 			{
-				_T("34-asian date file"),
+				L"34-asian date file",
 				2096,
-				R(_T("-r--r--r--")),
-				R(_T("root root")),
+				R(L"-r--r--r--"),
+				R(L"root root"),
 				0,
 				O(),
 				fz::datetime(fz::datetime::utc, calcYear(8, 17), 8, 17, 8, 52)
@@ -620,10 +623,10 @@ void CDirectoryListingParserTest::InitEntries()
 	m_entries.emplace_back(t_entry({
 			"-r-xr-xr-x   2 root  root  96 2004.07.15   35-dotted-date file",
 			{
-				_T("35-dotted-date file"),
+				L"35-dotted-date file",
 				96,
-				R(_T("-r-xr-xr-x")),
-				R(_T("root root")),
+				R(L"-r-xr-xr-x"),
+				R(L"root root"),
 				0,
 				O(),
 				fz::datetime(fz::datetime::utc, 2004, 7, 15)
@@ -637,10 +640,10 @@ void CDirectoryListingParserTest::InitEntries()
 	m_entries.emplace_back(t_entry({
 			"36-vms-dir.DIR;1  1 19-NOV-2001 21:41 [root,root] (RWE,RWE,RE,RE)",
 			{
-				_T("36-vms-dir"),
+				L"36-vms-dir",
 				512,
-				R(_T("RWE,RWE,RE,RE")),
-				R(_T("root,root")),
+				R(L"RWE,RWE,RE,RE"),
+				R(L"root,root"),
 				CDirentry::flag_dir,
 				O(),
 				fz::datetime(fz::datetime::utc, 2001, 11, 19, 21, 41)
@@ -652,7 +655,7 @@ void CDirectoryListingParserTest::InitEntries()
 	m_entries.emplace_back(t_entry({
 			"37-vms-file;1       155   2-JUL-2003 10:30:13.64",
 			{
-				_T("37-vms-file;1"),
+				L"37-vms-file;1",
 				79360,
 				R(),
 				R(),
@@ -667,10 +670,10 @@ void CDirectoryListingParserTest::InitEntries()
 	m_entries.emplace_back(t_entry({
 			"38-vms-notime-file;1    2/8    7-JAN-2000    [IV2_XXX]   (RWED,RWED,RE,)",
 			{
-				_T("38-vms-notime-file;1"),
+				L"38-vms-notime-file;1",
 				1024,
-				R(_T("RWED,RWED,RE,")),
-				R(_T("IV2_XXX")),
+				R(L"RWED,RWED,RE,"),
+				R(L"IV2_XXX"),
 				0,
 				O(),
 				fz::datetime(fz::datetime::utc, 2000, 1, 7)
@@ -682,10 +685,10 @@ void CDirectoryListingParserTest::InitEntries()
 	m_entries.emplace_back(t_entry({
 			"39-vms-notime-file;1    6/8    15-JUI-2002    PRONAS   (RWED,RWED,RE,)",
 			{
-				_T("39-vms-notime-file;1"),
+				L"39-vms-notime-file;1",
 				3072,
-				R(_T("RWED,RWED,RE,")),
-				R(_T("PRONAS")),
+				R(L"RWED,RWED,RE,"),
+				R(L"PRONAS"),
 				0,
 				O(),
 				fz::datetime(fz::datetime::utc, 2002, 7, 15)
@@ -696,10 +699,10 @@ void CDirectoryListingParserTest::InitEntries()
 	m_entries.emplace_back(t_entry({
 			"40-vms-multiline-file;1\r\n170774/170775     24-APR-2003 08:16:15  [FTP_CLIENT,SCOT]      (RWED,RWED,RE,)",
 			{
-				_T("40-vms-multiline-file;1"),
+				L"40-vms-multiline-file;1",
 				87436288,
-				R(_T("RWED,RWED,RE,")),
-				R(_T("FTP_CLIENT,SCOT")),
+				R(L"RWED,RWED,RE,"),
+				R(L"FTP_CLIENT,SCOT"),
 				0,
 				O(),
 				fz::datetime(fz::datetime::utc, 2003, 4, 24, 8, 16, 15)
@@ -710,10 +713,10 @@ void CDirectoryListingParserTest::InitEntries()
 	m_entries.emplace_back(t_entry({
 			"41-vms-multiline-file;1\r\n10     2-JUL-2003 10:30:08.59  [FTP_CLIENT,SCOT]      (RWED,RWED,RE,)",
 			{
-				_T("41-vms-multiline-file;1"),
+				L"41-vms-multiline-file;1",
 				5120,
-				R(_T("RWED,RWED,RE,")),
-				R(_T("FTP_CLIENT,SCOT")),
+				R(L"RWED,RWED,RE,"),
+				R(L"FTP_CLIENT,SCOT"),
 				0,
 				O(),
 				fz::datetime(fz::datetime::utc, 2003, 7, 2, 10, 30, 8)
@@ -725,10 +728,10 @@ void CDirectoryListingParserTest::InitEntries()
 	m_entries.emplace_back(t_entry({
 			"42-vms-alternate-field-order-file;1   [SUMMARY]    1/3     2-AUG-2006 13:05  (RWE,RWE,RE,)",
 			{
-				_T("42-vms-alternate-field-order-file;1"),
+				L"42-vms-alternate-field-order-file;1",
 				512,
-				R(_T("RWE,RWE,RE,")),
-				R(_T("SUMMARY")),
+				R(L"RWE,RWE,RE,"),
+				R(L"SUMMARY"),
 				0,
 				O(),
 				fz::datetime(fz::datetime::utc, 2006, 8, 2, 13, 5)
@@ -739,9 +742,9 @@ void CDirectoryListingParserTest::InitEntries()
 	m_entries.emplace_back(t_entry({
 			"43-vms-alternate-field-order-file;1       17-JUN-1994 17:25:37     6308/13     (RWED,RWED,R,)",
 			{
-				_T("43-vms-alternate-field-order-file;1"),
+				L"43-vms-alternate-field-order-file;1",
 				3229696,
-				R(_T("RWED,RWED,R,")),
+				R(L"RWED,RWED,R,"),
 				R(),
 				0,
 				O(),
@@ -757,10 +760,10 @@ void CDirectoryListingParserTest::InitEntries()
 	m_entries.emplace_back(t_entry({
 			"QSYS            77824 02/23/00 15:09:55 *DIR 44-ibm-as400 dir/",
 			{
-				_T("44-ibm-as400 dir"),
+				L"44-ibm-as400 dir",
 				77824,
 				R(),
-				R(_T("QSYS")),
+				R(L"QSYS"),
 				CDirentry::flag_dir | 0,
 				O(),
 				fz::datetime(fz::datetime::utc, 2000, 2, 23, 15, 9, 55)
@@ -771,10 +774,10 @@ void CDirectoryListingParserTest::InitEntries()
 	m_entries.emplace_back(t_entry({
 			"QSYS            77824 23/02/00 15:09:55 *FILE 45-ibm-as400-date file",
 			{
-				_T("45-ibm-as400-date file"),
+				L"45-ibm-as400-date file",
 				77824,
 				R(),
-				R(_T("QSYS")),
+				R(L"QSYS"),
 				0,
 				O(),
 				fz::datetime(fz::datetime::utc, 2000, 2, 23, 15, 9, 55)
@@ -786,10 +789,10 @@ void CDirectoryListingParserTest::InitEntries()
 	m_entries.emplace_back(t_entry({
 			"-r-xr-xr-x longowner longgroup123456 Feb 12 17:20 46-unix-concatsize file",
 			{
-				_T("46-unix-concatsize file"),
+				L"46-unix-concatsize file",
 				123456,
-				R(_T("-r-xr-xr-x")),
-				R(_T("longowner longgroup")),
+				R(L"-r-xr-xr-x"),
+				R(L"longowner longgroup"),
 				0,
 				O(),
 				fz::datetime(fz::datetime::utc, calcYear(2, 12), 2, 12, 17, 20)
@@ -801,10 +804,10 @@ void CDirectoryListingParserTest::InitEntries()
 	m_entries.emplace_back(t_entry({
 			"-r-xr-xr-x 2 owner group 4512 01-jun-99 47_unix_shortdatemonth file",
 			{
-				_T("47_unix_shortdatemonth file"),
+				L"47_unix_shortdatemonth file",
 				4512,
-				R(_T("-r-xr-xr-x")),
-				R(_T("owner group")),
+				R(L"-r-xr-xr-x"),
+				R(L"owner group"),
 				0,
 				O(),
 				fz::datetime(fz::datetime::utc, 1999, 6, 1)
@@ -816,7 +819,7 @@ void CDirectoryListingParserTest::InitEntries()
 	m_entries.emplace_back(t_entry({
 			"48-nortel-wfftp-file       1014196  06/03/04  Thur.   10:20:03",
 			{
-				_T("48-nortel-wfftp-file"),
+				L"48-nortel-wfftp-file",
 				1014196,
 				R(),
 				R(),
@@ -831,7 +834,7 @@ void CDirectoryListingParserTest::InitEntries()
 	m_entries.emplace_back(t_entry({
 			"2048    Feb-28-1998  05:23:30   49-nortel-vxworks dir <DIR>",
 			{
-				_T("49-nortel-vxworks dir"),
+				L"49-nortel-vxworks dir",
 				2048,
 				R(),
 				R(),
@@ -846,10 +849,10 @@ void CDirectoryListingParserTest::InitEntries()
 	m_entries.emplace_back(t_entry({
 			"-C--E-----FTP B BCC3I1       7670  1294495 Jan 13 07:42 50-conent file",
 			{
-				_T("50-conent file"),
+				L"50-conent file",
 				1294495,
-				R(_T("-C--E-----FTP")),
-				R(_T("B BCC3I1 7670")),
+				R(L"-C--E-----FTP"),
+				R(L"B BCC3I1 7670"),
 				0,
 				O(),
 				fz::datetime(fz::datetime::utc, calcYear(1, 13), 1, 13, 7, 42)
@@ -862,10 +865,10 @@ void CDirectoryListingParserTest::InitEntries()
 	m_entries.emplace_back(t_entry({
 			"20.20 07/03/29 1026 d-ewrewr 2650 85920 51-OS-9 dir",
 			{
-				_T("51-OS-9 dir"),
+				L"51-OS-9 dir",
 				85920,
-				R(_T("d-ewrewr")),
-				R(_T("20.20")),
+				R(L"d-ewrewr"),
+				R(L"20.20"),
 				CDirentry::flag_dir,
 				O(),
 				fz::datetime(fz::datetime::utc, 2007, 3, 29)
@@ -877,10 +880,10 @@ void CDirectoryListingParserTest::InitEntries()
 	m_entries.emplace_back(t_entry({
 			"drwxr-xr-x 3 user group 512 01 oct 2004 52-swapped-daymonth dir",
 			{
-				_T("52-swapped-daymonth dir"),
+				L"52-swapped-daymonth dir",
 				512,
-				R(_T("drwxr-xr-x")),
-				R(_T("user group")),
+				R(L"drwxr-xr-x"),
+				R(L"user group"),
 				CDirentry::flag_dir,
 				O(),
 				fz::datetime(fz::datetime::utc, 2004, 10, 1)
@@ -891,9 +894,9 @@ void CDirectoryListingParserTest::InitEntries()
 	m_entries.emplace_back(t_entry({
 			"-r--r--r-- 0125039 12 Nov 11 2005 53-noownergroup file",
 			{
-				_T("53-noownergroup file"),
+				L"53-noownergroup file",
 				12,
-				R(_T("-r--r--r--")),
+				R(L"-r--r--r--"),
 				R(),
 				0,
 				O(),
@@ -906,10 +909,10 @@ void CDirectoryListingParserTest::InitEntries()
 			// Valid UTF-8 encoding
 			"drwxr-xr-x   5 root     sys          512 2005\xEB\x85\x84  1\xEC\x9B\x94  6\xEC\x9D\xBC 54-asian date year first dir",
 			{
-				_T("54-asian date year first dir"),
+				L"54-asian date year first dir",
 				512,
-				R(_T("drwxr-xr-x")),
-				R(_T("root sys")),
+				R(L"drwxr-xr-x"),
+				R(L"root sys"),
 				CDirentry::flag_dir,
 				O(),
 				fz::datetime(fz::datetime::utc, 2005, 1, 6)
@@ -921,10 +924,10 @@ void CDirectoryListingParserTest::InitEntries()
 	m_entries.emplace_back(t_entry({
 			"QPGMR           36864 18.09.06 14:21:26 *FILE      55-AS400.FILE",
 			{
-				_T("55-AS400.FILE"),
+				L"55-AS400.FILE",
 				36864,
 				R(),
-				R(_T("QPGMR")),
+				R(L"QPGMR"),
 				0,
 				O(),
 				fz::datetime(fz::datetime::utc, 2006, 9, 18, 14, 21, 26)
@@ -936,7 +939,7 @@ void CDirectoryListingParserTest::InitEntries()
 	m_entries.emplace_back(t_entry({
 			"56-VMS-complex-size;1 2KB 23-SEP-2005 14:57:07.27",
 			{
-				_T("56-VMS-complex-size;1"),
+				L"56-VMS-complex-size;1",
 				2048,
 				R(),
 				R(),
@@ -951,10 +954,10 @@ void CDirectoryListingParserTest::InitEntries()
 	m_entries.emplace_back(t_entry({
 			"57-HP_NonStop 101 528 6-Apr-07 14:21:18 255, 0 \"oooo\"",
 			{
-				_T("57-HP_NonStop"),
+				L"57-HP_NonStop",
 				528,
-				R(_T("\"oooo\"")),
-				R(_T("255, 0")),
+				R(L"\"oooo\""),
+				R(L"255, 0"),
 				0,
 				O(),
 				fz::datetime(fz::datetime::utc, 2007, 4, 6, 14, 21, 18)
@@ -966,10 +969,10 @@ void CDirectoryListingParserTest::InitEntries()
 	m_entries.emplace_back(t_entry({
 			"58-HP_NonStop 101 528 6-Apr-07 14:21:18 255,255 \"oooo\"",
 			{
-				_T("58-HP_NonStop"),
+				L"58-HP_NonStop",
 				528,
-				R(_T("\"oooo\"")),
-				R(_T("255,255")),
+				R(L"\"oooo\""),
+				R(L"255,255"),
 				0,
 				O(),
 				fz::datetime(fz::datetime::utc, 2007, 4, 6, 14, 21, 18)
@@ -981,10 +984,10 @@ void CDirectoryListingParserTest::InitEntries()
 	m_entries.emplace_back(t_entry({
 			"drwxr-xr-x 6 user sys 1024 30. Jan., 12:40 59-localized-date-dir",
 			{
-				_T("59-localized-date-dir"),
+				L"59-localized-date-dir",
 				1024,
-				R(_T("drwxr-xr-x")),
-				R(_T("user sys")),
+				R(L"drwxr-xr-x"),
+				R(L"user sys"),
 				CDirentry::flag_dir,
 				O(),
 				fz::datetime(fz::datetime::utc, calcYear(1, 30), 1, 30, 12, 40)
@@ -1004,7 +1007,7 @@ void CDirectoryListingParserTest::InitEntries()
 	m_entries.emplace_back(t_entry({
 			"WYOSPT 3420   2003/05/21  1  200  FB      80  8053  PS  60-MVS.FILE",
 			{
-				_T("60-MVS.FILE"),
+				L"60-MVS.FILE",
 				100,
 				R(),
 				R(),
@@ -1018,7 +1021,7 @@ void CDirectoryListingParserTest::InitEntries()
 	m_entries.emplace_back(t_entry({
 			"WPTA01 3290   2004/03/04  1    3  FB      80  3125  PO  61-MVS.DATASET",
 			{
-				_T("61-MVS.DATASET"),
+				L"61-MVS.DATASET",
 				-1,
 				R(),
 				R(),
@@ -1032,7 +1035,7 @@ void CDirectoryListingParserTest::InitEntries()
 	m_entries.emplace_back(t_entry({
 			"NRP004 3390   **NONE**    1   15  NONE     0     0  PO  62-MVS-NONEDATE.DATASET",
 			{
-				_T("62-MVS-NONEDATE.DATASET"),
+				L"62-MVS-NONEDATE.DATASET",
 				-1,
 				R(),
 				R(),
@@ -1046,7 +1049,7 @@ void CDirectoryListingParserTest::InitEntries()
 	m_entries.emplace_back(t_entry({
 			"TSO005 3390   2005/06/06 213000 U 0 27998 PO 63-MVS.DATASET",
 			{
-				_T("63-MVS.DATASET"),
+				L"63-MVS.DATASET",
 				-1,
 				R(),
 				R(),
@@ -1060,7 +1063,7 @@ void CDirectoryListingParserTest::InitEntries()
 	m_entries.emplace_back(t_entry({
 			"TSO004 3390   VSAM 64-mvs-file",
 			{
-				_T("64-mvs-file"),
+				L"64-mvs-file",
 				-1,
 				R(),
 				R(),
@@ -1081,7 +1084,7 @@ void CDirectoryListingParserTest::InitEntries()
 	m_entries.emplace_back(t_entry({
 			"65-MVS-PDS-MEMBER",
 			{
-				_T("65-MVS-PDS-MEMBER"),
+				L"65-MVS-PDS-MEMBER",
 				-1,
 				R(),
 				R(),
@@ -1096,7 +1099,7 @@ void CDirectoryListingParserTest::InitEntries()
 	m_entries.emplace_back(t_entry({
 			"66-MVSPDSMEMBER 01.01 2004/06/22 2004/06/22 16:32   128   128    0 BOBY12",
 			{
-				_T("66-MVSPDSMEMBER"),
+				L"66-MVSPDSMEMBER",
 				128,
 				R(),
 				R(),
@@ -1111,7 +1114,7 @@ void CDirectoryListingParserTest::InitEntries()
 	m_entries.emplace_back(t_entry({
 			"67-MVSPDSMEMBER2 00B308 000411  00 FO                31    ANY",
 			{
-				_T("67-MVSPDSMEMBER2"),
+				L"67-MVSPDSMEMBER2",
 				45832,
 				R(),
 				R(),
@@ -1125,7 +1128,7 @@ void CDirectoryListingParserTest::InitEntries()
 	m_entries.emplace_back(t_entry({
 			"68-MVSPDSMEMBER3 00B308 000411  00 FO        RU      ANY    24",
 			{
-				_T("68-MVSPDSMEMBER3"),
+				L"68-MVSPDSMEMBER3",
 				45832,
 				R(),
 				R(),
@@ -1140,7 +1143,7 @@ void CDirectoryListingParserTest::InitEntries()
 	m_entries.emplace_back(t_entry({
 			"Migrated				69-SOME.FILE",
 			{
-				_T("69-SOME.FILE"),
+				L"69-SOME.FILE",
 				-1,
 				R(),
 				R(),
@@ -1172,10 +1175,10 @@ void CDirectoryListingParserTest::InitEntries()
 	m_entries.emplace_back(t_entry({
 			"70-ZVMFILE  TRACE   V        65      107        2 2005-10-04 15:28:42 060191",
 			{
-				_T("70-ZVMFILE.TRACE"),
+				L"70-ZVMFILE.TRACE",
 				6955,
 				R(),
-				R(_T("060191")),
+				R(L"060191"),
 				0,
 				O(),
 				fz::datetime(fz::datetime::utc, 2005, 10, 4, 15, 28, 42)
@@ -1186,10 +1189,10 @@ void CDirectoryListingParserTest::InitEntries()
 	m_entries.emplace_back(t_entry({
 			"drwxr-xr-x 3 slopri devlab 512 71-unix-dateless",
 			{
-				_T("71-unix-dateless"),
+				L"71-unix-dateless",
 				512,
-				R(_T("drwxr-xr-x")),
-				R(_T("slopri devlab")),
+				R(L"drwxr-xr-x"),
+				R(L"slopri devlab"),
 				CDirentry::flag_dir,
 				O(),
 				fz::datetime()
@@ -1200,7 +1203,7 @@ void CDirectoryListingParserTest::InitEntries()
 	m_entries.emplace_back(t_entry({
 			"Type=file;mOdIfY=20081105165215;size=1234; 72-MLSD-file",
 			{
-				_T("72-MLSD-file"),
+				L"72-MLSD-file",
 				1234,
 				R(),
 				R(),
@@ -1218,7 +1221,7 @@ void CDirectoryListingParserTest::InitEntries()
 	m_entries.emplace_back(t_entry({
 			"V43525 Tape                                             73-MSV-TAPE.FILE",
 			{
-				_T("73-MSV-TAPE.FILE"),
+				L"73-MSV-TAPE.FILE",
 				-1,
 				R(),
 				R(),
@@ -1232,7 +1235,7 @@ void CDirectoryListingParserTest::InitEntries()
 	m_entries.emplace_back(t_entry({
 			"Type=file; 74-MLSD-whitespace trailing\t ",
 			{
-				_T("74-MLSD-whitespace trailing\t "),
+				L"74-MLSD-whitespace trailing\t ",
 				-1,
 				R(),
 				R(),
@@ -1246,7 +1249,7 @@ void CDirectoryListingParserTest::InitEntries()
 		m_entries.emplace_back(t_entry({
 			"Type=file; \t 75-MLSD-whitespace leading",
 			{
-				_T("\t 75-MLSD-whitespace leading"),
+				L"\t 75-MLSD-whitespace leading",
 				-1,
 				R(),
 				R(),
@@ -1260,10 +1263,10 @@ void CDirectoryListingParserTest::InitEntries()
 		m_entries.emplace_back(t_entry({
 			"modify=20080426135501;perm=;size=65718921;type=file;unique=802U1066013B;UNIX.group=1179;UNIX.mode=00;UNIX.owner=1179; 75 MLSD file with empty permissions",
 			{
-				_T("75 MLSD file with empty permissions"),
+				L"75 MLSD file with empty permissions",
 				65718921,
-				R(_T("00")),
-				R(_T("1179 1179")),
+				R(L"00"),
+				R(L"1179 1179"),
 				0,
 				O(),
 				fz::datetime(fz::datetime::utc, 2008, 4, 26, 13, 55, 1)
@@ -1274,12 +1277,12 @@ void CDirectoryListingParserTest::InitEntries()
 		m_entries.emplace_back(t_entry({
 			"type=OS.unix=slink:/foo; 76 MLSD symlink",
 			{
-				_T("76 MLSD symlink"),
+				L"76 MLSD symlink",
 				-1,
 				R(),
 				R(),
 				CDirentry::flag_dir | CDirentry::flag_link,
-				O(_T("/foo")),
+				O(L"/foo"),
 				fz::datetime()
 			},
 			DEFAULT
@@ -1288,7 +1291,7 @@ void CDirectoryListingParserTest::InitEntries()
 		m_entries.emplace_back(t_entry({
 			"type=OS.UNIX=symlink; 76b MLSD symlink",
 			{
-				_T("76b MLSD symlink"),
+				L"76b MLSD symlink",
 				-1,
 				R(),
 				R(),
@@ -1303,7 +1306,7 @@ void CDirectoryListingParserTest::InitEntries()
 		m_entries.emplace_back(t_entry({
 			"type=file 77 MLSD file no trailing semicolon after facts < mlst-07",
 			{
-				_T("77 MLSD file no trailing semicolon after facts < mlst-07"),
+				L"77 MLSD file no trailing semicolon after facts < mlst-07",
 				-1,
 				R(),
 				R(),
@@ -1317,7 +1320,7 @@ void CDirectoryListingParserTest::InitEntries()
 		m_entries.emplace_back(t_entry({
 			"type=OS.unix=slink; 77 MLSD symlink notarget",
 			{
-				_T("77 MLSD symlink notarget"),
+				L"77 MLSD symlink notarget",
 				-1,
 				R(),
 				R(),
@@ -1331,7 +1334,7 @@ void CDirectoryListingParserTest::InitEntries()
 	m_entries.emplace_back(t_entry({
 		"size=1365694195;type=file;modify=20090722092510;\tadsl TV 2009-07-22 08-25-10 78 mlsd file that can get parsed as unix.file",
 		{
-			_T("adsl TV 2009-07-22 08-25-10 78 mlsd file that can get parsed as unix.file"),
+			L"adsl TV 2009-07-22 08-25-10 78 mlsd file that can get parsed as unix.file",
 			1365694195,
 			R(),
 			R(),
@@ -1347,7 +1350,7 @@ void CDirectoryListingParserTest::InitEntries()
 	m_entries.emplace_back(t_entry({
 		"WYOSPT 3420   2003/05/21  1 ????  FB      80  8053  PS  79-MVS.FILE",
 		{
-			_T("79-MVS.FILE"),
+			L"79-MVS.FILE",
 			100,
 			R(),
 			R(),
@@ -1364,7 +1367,7 @@ void CDirectoryListingParserTest::InitEntries()
 	m_entries.emplace_back(t_entry({
 		"GISBWI 3390   2011/08/25  2 ++++  FB     904 18080  PS  80-MVS.FILE",
 		{
-			_T("80-MVS.FILE"),
+			L"80-MVS.FILE",
 			100,
 			R(),
 			R(),
@@ -1381,7 +1384,7 @@ void CDirectoryListingParserTest::InitEntries()
 	m_entries.emplace_back(t_entry({
 		"WYOSPT 3420   2003/05/21  1 3 U 6447    6447  PO-E 81-MVS.DIR",
 		{
-			_T("81-MVS.DIR"),
+			L"81-MVS.DIR",
 			-1,
 			R(),
 			R(),
@@ -1395,10 +1398,10 @@ void CDirectoryListingParserTest::InitEntries()
 	m_entries.push_back(t_entry({
 		"drwxrwxrwx   1 0        0               0 29 Jul 02:27 2014 Invoices",
 		{
-			_T("2014 Invoices"),
+			L"2014 Invoices",
 			0,
-			R(_T("drwxrwxrwx")),
-			R(_T("0 0")),
+			R(L"drwxrwxrwx"),
+			R(L"0 0"),
 			CDirentry::flag_dir,
 			O(),
 			fz::datetime(fz::datetime::utc, calcYear(7, 29), 7, 29, 2, 27)
@@ -1410,7 +1413,7 @@ void CDirectoryListingParserTest::InitEntries()
 	m_entries.emplace_back(t_entry({
 		"Type=file;mOdIfY=19681105165215;size=1234; MLSD pre-epoch",
 		{
-			_T("MLSD pre-epoch"),
+			L"MLSD pre-epoch",
 			1234,
 			R(),
 			R(),
@@ -1424,10 +1427,10 @@ void CDirectoryListingParserTest::InitEntries()
 	m_entries.emplace_back(t_entry({
 		"-rw-------      1  99999999 0              3 Apr   4 24:00 alternate_midnight",
 		{
-			_T("alternate_midnight"),
+			L"alternate_midnight",
 			3,
-			R(_T("-rw-------")),
-			R(_T("99999999 0")),
+			R(L"-rw-------"),
+			R(L"99999999 0"),
 			0,
 			O(),
 			fz::datetime(fz::datetime::utc, calcYear(4, 4), 4, 5, 0, 0)
@@ -1473,15 +1476,14 @@ void CDirectoryListingParserTest::testIndividual()
 
 	CDirectoryListing listing = parser.Parse(CServerPath());
 
-	wxString msg = wxString::Format(_T("Data: %s, count: %d"), wxString(entry.data.c_str(), wxConvUTF8).c_str(), listing.GetCount());
-	msg.Replace(_T("\r"), _T(""));
-	msg.Replace(_T("\n"), _T(""));
-	wxWX2MBbuf mb_buf = msg.mb_str(wxConvUTF8);
-	CPPUNIT_ASSERT_MESSAGE((const char*)mb_buf, listing.GetCount() == 1);
+	std::string msg = fz::sprintf("Data: %s, count: %d", entry.data, listing.GetCount());
+	fz::replace_substrings(msg, "\r", std::string());
+	fz::replace_substrings(msg, "\n", std::string());
 
-	msg = wxString::Format(_T("Data: %s  Expected:\n%s\n  Got:\n%s"), wxString(entry.data.c_str(), wxConvUTF8).c_str(), entry.reference.dump().c_str(), listing[0].dump().c_str());
-	mb_buf = msg.mb_str(wxConvUTF8);
-	CPPUNIT_ASSERT_MESSAGE((const char*)mb_buf, listing[0] == entry.reference);
+	CPPUNIT_ASSERT_MESSAGE(msg, listing.GetCount() == 1);
+
+	msg = fz::sprintf("Data: %s  Expected:\n%s\n  Got:\n%s", entry.data, entry.reference.dump(), listing[0].dump());
+	CPPUNIT_ASSERT_MESSAGE(msg, listing[0] == entry.reference);
 }
 
 void CDirectoryListingParserTest::testAll()
@@ -1501,11 +1503,10 @@ void CDirectoryListingParserTest::testAll()
 	CPPUNIT_ASSERT(listing.GetCount() == m_entries.size());
 
 	unsigned int i = 0;
-	for (std::vector<t_entry>::const_iterator iter = m_entries.begin(); iter != m_entries.end(); iter++, i++)
-	{
-		wxString msg = wxString::Format(_T("Data: %s  Expected:\n%s\n  Got:\n%s"), wxString(iter->data.c_str(), wxConvUTF8).c_str(), iter->reference.dump().c_str(), listing[i].dump().c_str());
+	for (auto iter = m_entries.begin(); iter != m_entries.end(); iter++, i++) {
+		std::string msg = fz::sprintf("Data: %s  Expected:\n%s\n  Got:\n%s", iter->data, iter->reference.dump(), listing[i].dump());
 
-		CPPUNIT_ASSERT_MESSAGE((const char*)msg.mb_str(wxConvUTF8), listing[i] == iter->reference);
+		CPPUNIT_ASSERT_MESSAGE(msg, listing[i] == iter->reference);
 	}
 }
 
